@@ -35,6 +35,8 @@
 #include <QBuffer>
 #include <QPainter>
 
+#include <qca_basic.h>
+
 #include "xmpp_xmlcommon.h"
 #include "xmpp_vcard.h"
 #include "xmpp_client.h"
@@ -56,10 +58,10 @@ using namespace QCA;
 
 //------------------------------------------------------------------------------
 
-static QByteArray scaleAvatar(const QByteArray& b)	
+static QByteArray scaleAvatar(const QByteArray& b)
 {
-	//int maxSize = (option.avatarsSize > MAX_AVATAR_SIZE ? MAX_AVATAR_SIZE : option.avatarsSize);
-	int maxSize = MAX_AVATAR_SIZE;
+	//int maxSize = (LEGOPTS.avatarsSize > MAX_AVATAR_SIZE ? MAX_AVATAR_SIZE : LEGOPTS.avatarsSize);
+	int maxSize = AvatarFactory::maxAvatarSize();
 	QImage i(b);
 	if (i.isNull()) {
 		qWarning("AvatarFactory::scaleAvatar(): Null image (unrecognized format?)");
@@ -145,7 +147,7 @@ protected:
 	
 	virtual bool isCached(const QString& hash);
 	virtual void loadFromCache(const QString& hash);
-	virtual void saveToCache(const QString& hash, const QByteArray& data);
+	virtual void saveToCache(const QByteArray& data);
 
 private:
 	QString hash_;
@@ -188,9 +190,9 @@ void CachedAvatar::loadFromCache(const QString& h)
 	}
 }
 
-void CachedAvatar::saveToCache(const QString& hash, const QByteArray& data)
+void CachedAvatar::saveToCache(const QByteArray& data)
 {
-	// Write file to cache
+	QString hash = QCA::Hash("sha1").hashToString(data);
 	// printf("Saving %s to cache.\n",hash.latin1());
 	QString fn = QDir(AvatarFactory::getCacheDir()).filePath(hash);
 	QFile f(fn);
@@ -222,7 +224,7 @@ public:
 		if (h == hash()) {
 			QByteArray ba = Base64().stringToArray(data).toByteArray();
 			if (!ba.isEmpty()) {
-				saveToCache(hash(),ba);
+				saveToCache(ba);
 				setImage(ba);
 				if (pixmap().isNull()) {
 					qWarning("PEPAvatar::setData(): Null pixmap. Unsupported format ?");
@@ -292,7 +294,7 @@ void VCardAvatar::receivedVCard()
 {
 	const VCard* vcard = VCardFactory::instance()->vcard(jid_);
 	if (vcard) {
-		saveToCache(hash(),vcard->photo());
+		saveToCache(vcard->photo());
 		setImage(vcard->photo());
 		emit avatarChanged(jid_);
 	}
@@ -476,8 +478,12 @@ inline static QPixmap ensureSquareAvatar(const QPixmap& original)
 	return square;
 }
 
-QPixmap AvatarFactory::getAvatar(const Jid& jid)
+QPixmap AvatarFactory::getAvatar(const Jid& _jid)
 {
+	// protect from race condition when caller gets
+	// deleted as result of avatarChanged() signal
+	Jid jid = _jid;
+
 	// Compute the avatar of the user
 	Avatar* av = retrieveAvatar(jid);
 
@@ -627,11 +633,18 @@ QString AvatarFactory::getCacheDir()
 	return avatars.path();
 }
 
+int AvatarFactory::maxAvatarSize()
+{
+	return MAX_AVATAR_SIZE;
+}
+
 void AvatarFactory::itemPublished(const Jid& jid, const QString& n, const PubSubItem& item)
 {
 	if (n == "http://www.xmpp.org/extensions/xep-0084.html#ns-data") {
 		if (item.payload().tagName() == "data") {
-			pep_avatars_[jid.bare()]->setData(item.id(),item.payload().text());
+			if (pep_avatars_.contains(jid.bare())) {
+				pep_avatars_[jid.bare()]->setData(item.id(),item.payload().text());
+			}
 		}
 		else {
 			qWarning("avatars.cpp: Unexpected item payload");
