@@ -45,7 +45,6 @@
 #include "psiaccount.h"
 #include "psicon.h"
 #include "busywidget.h"
-#include "common.h"
 #include "iconaction.h"
 #include "psiiconset.h"
 #include "psitooltip.h"
@@ -1430,15 +1429,16 @@ DiscoDlg::DiscoDlg(PsiAccount *pa, const Jid &jid, const QString &node)
 	: QDialog(0)
 {
 	setAttribute(Qt::WA_DeleteOnClose);
-  	setupUi(this);
+	setupUi(this);
 	// restore options
-	ck_autoItems->setChecked(option.discoItems);
-	ck_autoInfo->setChecked(option.discoInfo);
+	ck_autoItems->setChecked(PsiOptions::instance()->getOption("options.ui.service-discovery.automatically-get-items").toBool());
+	ck_autoInfo->setChecked(PsiOptions::instance()->getOption("options.ui.service-discovery.automatically-get-info").toBool());
 
 	// initialize
 	d = new Private(this, pa);
 	d->jid  = jid;
 	d->node = node;
+	d->data.pa->dialogRegister(this);
 
 	setWindowTitle(CAP(caption()));
 	setWindowIcon(PsiIconset::instance()->transportStatus("transport", STATUS_ONLINE).icon());
@@ -1467,11 +1467,12 @@ DiscoDlg::DiscoDlg(PsiAccount *pa, const Jid &jid, const QString &node)
 
 DiscoDlg::~DiscoDlg()
 {
+	d->data.pa->dialogUnregister(this);
 	delete d;
 
 	// save options
-	option.discoItems = ck_autoItems->isChecked();
-	option.discoInfo  = ck_autoInfo->isChecked();
+	PsiOptions::instance()->setOption("options.ui.service-discovery.automatically-get-items", (bool) ck_autoItems->isChecked());
+	PsiOptions::instance()->setOption("options.ui.service-discovery.automatically-get-info", (bool) ck_autoInfo->isChecked());
 }
 
 void DiscoDlg::doDisco(QString host, QString node)
