@@ -29,7 +29,7 @@
 #include <qtimer.h>
 #include <qtextstream.h>
 
-#include "profiles.h"
+#include "common.h"
 #include "applicationinfo.h"
 #include "psievent.h"
 #include "jidutil.h"
@@ -590,7 +590,7 @@ void EDBFlatFile::file_timeout()
 {
 	File *i = (File *)sender();
 	d->flist.remove(i);
-	delete i;
+	i->deleteLater();
 }
 
 
