@@ -65,11 +65,9 @@
 #include "stretchwidget.h"
 #include "mucmanager.h"
 #include "busywidget.h"
-#include "common.h"
 #include "msgmle.h"
 #include "iconwidget.h"
 #include "iconselect.h"
-#include "psitoolbar.h"
 #include "iconaction.h"
 #include "psitooltip.h"
 #include "psioptions.h"
@@ -102,9 +100,7 @@ public:
 
 	GCMainDlg *dlg;
 	int state;
-	PsiAccount *pa;
 	MUCManager *mucManager;
-	Jid jid;
 	QString self, prev_self;
 	QString password;
 	bool nonAnonymous;     // got status code 100 ?
@@ -117,9 +113,6 @@ public:
 	int pending;
 	bool connecting;
 
-	QTimer *flashTimer;
-	int flashCount;
-
 	QStringList hist;
 	int histAt;
 
@@ -139,26 +132,21 @@ private:
 
 public slots:
 	void addEmoticon(const PsiIcon *icon) {
-		if ( !dlg->isActiveWindow() )
-		     return;
-
-		QString text;
-
-		QHash<QString,QString> itext = icon->text();
-		for ( QHash<QString,QString>::ConstIterator it = itext.begin(); it != itext.end(); ++it) {
-			if (  !it->isEmpty() ) {
-				text = (*it) + " ";
-				break;
-			}
+		if ( !dlg->isActiveTab() ) {
+			return;
 		}
 
-		if ( !text.isEmpty() )
-			mle()->insert( text );
+		QString text = icon->defaultText();
+
+		if (!text.isEmpty()) {
+			mle()->insert(text + " ");
+		}
 	}
 
 	void addEmoticon(QString text) {
-		if ( !dlg->isActiveWindow() )
-		     return;
+		if ( !dlg->isActiveTab() ) {
+			return;
+		}
 
 		mle()->insert( text + " " );
 	}
@@ -461,25 +449,22 @@ public:
 	}
 };
 
-GCMainDlg::GCMainDlg(PsiAccount *pa, const Jid &j)
-	: AdvancedWidget<QWidget>(0)
+GCMainDlg::GCMainDlg(PsiAccount *pa, const Jid &j, TabManager *tabManager)
+	: TabbableWidget(j.userHost(), pa, tabManager)
 {
 	setAttribute(Qt::WA_DeleteOnClose);
-  	if ( option.brushedMetal )
+  	if ( PsiOptions::instance()->getOption("options.ui.mac.use-brushed-metal-windows").toBool() )
 		setAttribute(Qt::WA_MacMetalStyle);
 	nicknumber=0;
 	d = new Private(this);
-	d->pa = pa;
-	d->jid = j.userHost();
 	d->self = d->prev_self = j.resource();
-	d->pa->dialogRegister(this, d->jid);
-	connect(d->pa, SIGNAL(updatedActivity()), SLOT(pa_updatedActivity()));
-	d->mucManager = new MUCManager(d->pa->client(),d->jid);
+	account()->dialogRegister(this, jid());
+	connect(account(), SIGNAL(updatedActivity()), SLOT(pa_updatedActivity()));
+	d->mucManager = new MUCManager(account()->client(), jid());
 
 	options_ = PsiOptions::instance();
 
 	d->pending = 0;
-	d->flashTimer = 0;
 	d->connecting = false;
 
 	d->histAt = 0;
@@ -495,13 +480,13 @@ GCMainDlg::GCMainDlg(PsiAccount *pa, const Jid &j)
 #endif
 
 	ui_.setupUi(this);
-	ui_.lb_ident->setAccount(d->pa);
+	ui_.lb_ident->setAccount(account());
 	ui_.lb_ident->setShowJid(false);
 
 	connect(ui_.pb_topic, SIGNAL(clicked()), SLOT(doTopic()));
 	PsiToolTip::install(ui_.le_topic);
 
-	connect(d->pa->psi(), SIGNAL(accountCountChanged()), this, SLOT(updateIdentityVisibility()));
+	connect(account()->psi(), SIGNAL(accountCountChanged()), this, SLOT(updateIdentityVisibility()));
 	updateIdentityVisibility();
 
 	d->act_find = new IconAction(tr("Find"), "psi/search", tr("&Find"), 0, this);
@@ -585,30 +570,45 @@ GCMainDlg::GCMainDlg(PsiAccount *pa, const Jid &j)
 
 	setLooks();
 	setShortcuts();
-	updateCaption();
+	invalidateTab();
 	setConnecting();
 }
 
 GCMainDlg::~GCMainDlg()
 {
 	if(d->state != Private::Idle)
-		d->pa->groupChatLeave(d->jid.host(), d->jid.user());
+		account()->groupChatLeave(jid().host(), jid().user());
 
 	//QMimeSourceFactory *m = ui_.log->mimeSourceFactory();
 	//ui_.log->setMimeSourceFactory(0);
 	//delete m;
 
-	d->pa->dialogUnregister(this);
+	account()->dialogUnregister(this);
 	delete d->mucManager;
 	delete d;
 }
 
+void GCMainDlg::ensureTabbedCorrectly() {
+	TabbableWidget::ensureTabbedCorrectly();
+	setShortcuts();
+	// QSplitter is broken again, force resize so that
+	// lv_users gets initizalised properly and context menu
+	// works in tabs too.
+	QList<int> tmp = ui_.hsplitter->sizes();
+	ui_.hsplitter->setSizes(QList<int>() << 0);
+	ui_.hsplitter->setSizes(tmp);
+}
+
 void GCMainDlg::setShortcuts()
 {
 	d->act_clear->setShortcuts(ShortcutManager::instance()->shortcuts("chat.clear"));
 	d->act_find->setShortcuts(ShortcutManager::instance()->shortcuts("chat.find"));
-	//d->act_send->setShortcuts(ShortcutManager::instance()->shortcuts("chat.send"));
-	//d->act_close->setShortcuts(ShortcutManager::instance()->shortcuts("common.close"));
+	d->act_send->setShortcuts(ShortcutManager::instance()->shortcuts("chat.send"));
+	if (!isTabbed()) {
+		d->act_close->setShortcuts(ShortcutManager::instance()->shortcuts("common.close"));
+	} else {
+		d->act_close->QAction::setShortcuts (QList<QKeySequence>());
+	}
 	d->act_scrollup->setShortcuts(ShortcutManager::instance()->shortcuts("common.scroll-up"));
 	d->act_scrolldown->setShortcuts(ShortcutManager::instance()->shortcuts("common.scroll-down"));
 }
@@ -621,23 +621,6 @@ void GCMainDlg::scrollDown() {
 	ui_.log->verticalScrollBar()->setValue(ui_.log->verticalScrollBar()->value() + ui_.log->verticalScrollBar()->pageStep()/2);
 }
 
-// FIXME: This should be unnecessary, since these keys are all registered as
-// actions in the constructor. Somehow, Qt ignores this.
-void GCMainDlg::keyPressEvent(QKeyEvent *e)
-{
-	QKeySequence key = e->key() + ( e->modifiers() & ~Qt::KeypadModifier);
-	if(!option.useTabs && ShortcutManager::instance()->shortcuts("common.close").contains(key))
-		close();
-	else if(ShortcutManager::instance()->shortcuts("chat.send").contains(key))
-		mle_returnPressed();
-	else if(ShortcutManager::instance()->shortcuts("common.scroll-up").contains(key))
-		scrollUp();
-	else if(ShortcutManager::instance()->shortcuts("common.scroll-down").contains(key))
-		scrollDown();
-	else
-		e->ignore();
-}
-
 void GCMainDlg::closeEvent(QCloseEvent *e)
 {
 	e->accept();
@@ -645,27 +628,29 @@ void GCMainDlg::closeEvent(QCloseEvent *e)
 
 void GCMainDlg::resizeEvent(QResizeEvent* e)
 {
-	if (option.keepSizes)
+	if (PsiOptions::instance()->getOption("options.ui.remember-window-sizes").toBool())
 		PsiOptions::instance()->setOption("options.ui.muc.size", e->size());
 }
 
-void GCMainDlg::windowActivationChange(bool oldstate)
+void GCMainDlg::deactivated()
 {
-	QWidget::windowActivationChange(oldstate);
+	TabbableWidget::deactivated();
 
-	// if we're bringing it to the front, get rid of the '*' if necessary
-	if(isActiveWindow()) {
-		if(d->pending > 0) {
-			d->pending = 0;
-			updateCaption();
-		}
-		doFlash(false);
+	d->trackBar = true;
+}
 
-		ui_.mle->chatEdit()->setFocus();
-		d->trackBar = false;
-	} else {
-		d->trackBar = true;
+void GCMainDlg::activated()
+{
+	TabbableWidget::activated();
+
+	if(d->pending > 0) {
+		d->pending = 0;
+		invalidateTab();
 	}
+	doFlash(false);
+
+	ui_.mle->chatEdit()->setFocus();
+	d->trackBar = false;
 }
 
 void GCMainDlg::mucInfoDialog(const QString& title, const QString& message, const Jid& actor, const QString& reason)
@@ -701,13 +686,13 @@ void GCMainDlg::setConnecting()
 
 void GCMainDlg::updateIdentityVisibility()
 {
-	ui_.lb_ident->setVisible(d->pa->psi()->contactList()->enabledAccounts().count() > 1);
+	ui_.lb_ident->setVisible(account()->psi()->contactList()->enabledAccounts().count() > 1);
 }
 
 #ifdef WHITEBOARDING
 void GCMainDlg::openWhiteboard()
 {
-	d->pa->actionOpenWhiteboardSpecific(d->jid, d->jid.withResource(d->self), true);
+	account()->actionOpenWhiteboardSpecific(jid(), jid().withResource(d->self), true);
 }
 #endif
 
@@ -738,10 +723,11 @@ void GCMainDlg::mle_returnPressed()
 
 	if(str.lower().startsWith("/nick ")) {
 		QString nick = str.mid(6).stripWhiteSpace();
-		if ( !nick.isEmpty() ) {
+		QString norm_nick;
+		if (!nick.isEmpty() && XMPP::Jid::validResource(nick, &norm_nick)) {
 			d->prev_self = d->self;
-			d->self = nick;
-			d->pa->groupChatChangeNick(d->jid.host(), d->jid.user(), d->self, d->pa->status());
+			d->self = norm_nick;
+			account()->groupChatChangeNick(jid().host(), jid().user(), d->self, account()->status());
 		}
 		ui_.mle->chatEdit()->setText("");
 		return;
@@ -750,7 +736,7 @@ void GCMainDlg::mle_returnPressed()
 	if(d->state != Private::Connected)
 		return;
 
-	Message m(d->jid);
+	Message m(jid());
 	m.setType("groupchat");
 	m.setBody(str);
 	m.setTimeStamp(QDateTime::currentDateTime());
@@ -790,7 +776,7 @@ void GCMainDlg::doTopic()
 		QLineEdit::Normal, ui_.le_topic->text(), &ok, this);
 
 	if(ok) {
-		Message m(d->jid);
+		Message m(jid());
 		m.setType("groupchat");
 		m.setSubject(str);
 		m.setTimeStamp(QDateTime::currentDateTime());
@@ -813,7 +799,7 @@ void GCMainDlg::doClearButton()
 void GCMainDlg::openFind()
 {
 	if(d->findDlg)
-		bringToFront(d->findDlg);
+		::bringToFront(d->findDlg);
 	else {
 		d->findDlg = new GCFindDlg(d->lastSearch, this);
 		connect(d->findDlg, SIGNAL(find(const QString &)), SLOT(doFind(const QString &)));
@@ -824,11 +810,13 @@ void GCMainDlg::openFind()
 void GCMainDlg::configureRoom()
 {
 	if(d->configDlg)
-		bringToFront(d->configDlg);
+		::bringToFront(d->configDlg);
 	else {
 		GCUserViewItem* c = (GCUserViewItem*)ui_.lv_users->findEntry(d->self);
+		MUCItem::Role role = c ? c->s.mucItem().role() : MUCItem::UnknownRole;
+		MUCItem::Affiliation affiliation = c ? c->s.mucItem().affiliation() : MUCItem::UnknownAffiliation;
 		d->configDlg = new MUCConfigDlg(d->mucManager, this);
-		d->configDlg->setRoleAffiliation(c->s.mucItem().role(), c->s.mucItem().affiliation());
+		d->configDlg->setRoleAffiliation(role, affiliation);
 		d->configDlg->show();
 	}
 }
@@ -858,11 +846,11 @@ void GCMainDlg::goConn()
 		d->state = Private::Connecting;
 		appendSysMsg(tr("Reconnecting..."), true);
 
-		QString host = d->jid.host();
-		QString room = d->jid.user();
+		QString host = jid().host();
+		QString room = jid().user();
 		QString nick = d->self;
 
-		if(!d->pa->groupChatJoin(host, room, nick, d->password)) {
+		if(!account()->groupChatJoin(host, room, nick, d->password)) {
 			appendSysMsg(tr("Error: You are in or joining this room already!"), true);
 			d->state = Private::Idle;
 		}
@@ -879,39 +867,36 @@ void GCMainDlg::dropEvent(QDropEvent *e)
 	Jid jid(e->mimeData()->text());
 	if (jid.isValid() && !ui_.lv_users->hasJid(jid)) {
 		Message m;
-		m.setTo(d->jid);
+		m.setTo(this->jid());
 		m.addMUCInvite(MUCInvite(jid));
 		if (!d->password.isEmpty())
 			m.setMUCPassword(d->password);
 		m.setTimeStamp(QDateTime::currentDateTime());
-		d->pa->dj_sendMessage(m);
+		account()->dj_sendMessage(m);
 	}
 }
 
 
 void GCMainDlg::pa_updatedActivity()
 {
-	if(!d->pa->loggedIn())
+	if(!account()->loggedIn()) {
 		goDisc();
+	}
 	else {
-		if(d->state == Private::Idle)
+		if(d->state == Private::Idle) {
 			goConn();
+		}
 		else if(d->state == Private::Connected) {
-			Status s = d->pa->status();
+			Status s = account()->status();
 			s.setXSigned("");
-			d->pa->groupChatSetStatus(d->jid.host(), d->jid.user(), s);
+			account()->groupChatSetStatus(jid().host(), jid().user(), s);
 		}
 	}
 }
 
-Jid GCMainDlg::jid() const
-{
-	return d->jid;
-}
-
 PsiAccount* GCMainDlg::account() const
 {
-	return d->pa;
+	return TabbableWidget::account();
 }
 
 void GCMainDlg::error(int, const QString &str)
@@ -1037,7 +1022,7 @@ void GCMainDlg::presence(const QString &nick, const Status &s)
 				message += tr("Do you want to join the alternate venue '%1' ?").arg(s.mucDestroy().jid().full());
 				int ret = QMessageBox::information(this, tr("Room Destroyed"), message, QMessageBox::Yes, QMessageBox::No);
 				if (ret == QMessageBox::Yes) {
-					d->pa->actionJoin(s.mucDestroy().jid().full());
+					account()->actionJoin(s.mucDestroy().jid().full());
 				}
 			}
 			else {
@@ -1135,8 +1120,8 @@ void GCMainDlg::presence(const QString &nick, const Status &s)
 	}
 	
 	if (!s.capsNode().isEmpty()) {
-		Jid caps_jid(s.mucItem().jid().isEmpty() || !d->nonAnonymous ? Jid(d->jid).withResource(nick) : s.mucItem().jid());
-		d->pa->capsManager()->updateCaps(caps_jid,s.capsNode(),s.capsVersion(),s.capsExt());
+		Jid caps_jid(s.mucItem().jid().isEmpty() || !d->nonAnonymous ? Jid(jid()).withResource(nick) : s.mucItem().jid());
+		account()->capsManager()->updateCaps(caps_jid,s.capsNode(),s.capsVersion(),s.capsExt());
 	}
 
 }
@@ -1170,9 +1155,10 @@ void GCMainDlg::message(const Message &_m)
 	if (m.body().left(d->self.length()) == d->self)
 		d->lastReferrer = m.from().resource();
 
-	if(option.gcHighlighting) {
-		for(QStringList::Iterator it=option.gcHighlights.begin();it!=option.gcHighlights.end();it++) {
-			if(m.body().contains((*it), Qt::CaseInsensitive)) {
+	if(PsiOptions::instance()->getOption("options.ui.muc.use-highlighting").toBool()) {
+		QStringList highlightWords = PsiOptions::instance()->getOption("options.ui.muc.highlight-words").toStringList();
+		foreach (QString word, highlightWords) {
+			if(m.body().contains((word), Qt::CaseInsensitive)) {
 				alert = true;
 			}
 		}
@@ -1181,11 +1167,11 @@ void GCMainDlg::message(const Message &_m)
 	// play sound?
 	if(from == d->self) {
 		if(!m.spooled())
-			d->pa->playSound(option.onevent[eSend]);
+			account()->playSound(PsiOptions::instance()->getOption("options.ui.notifications.sounds.outgoing-chat").toString());
 	}
 	else {
-		if(alert || (!option.noGCSound && !m.spooled() && !from.isEmpty()) )
-			d->pa->playSound(option.onevent[eChat2]);
+		if(alert || (PsiOptions::instance()->getOption("options.ui.notifications.sounds.notify-every-muc-message").toBool() && !m.spooled() && !from.isEmpty()) )
+			account()->playSound(PsiOptions::instance()->getOption("options.ui.notifications.sounds.chat-message").toString());
 	}
 
 	if(from.isEmpty())
@@ -1216,18 +1202,29 @@ const QString& GCMainDlg::nick() const
 	return d->self;
 }
 
+void GCMainDlg::updateLastMsgTime(QDateTime t)
+{
+	bool doInsert = t.date() != lastMsgTime_.date();
+	lastMsgTime_ = t;
+	if (doInsert) {
+		QString color = "#00A000";
+		ui_.log->appendText(QString("<font color=\"%1\">*** %2</font>").arg(color).arg(t.date().toString(Qt::ISODate)));
+	}
+}
+
 void GCMainDlg::appendSysMsg(const QString &str, bool alert, const QDateTime &ts)
 {
 	if (d->trackBar)
 	 	d->doTrackBar();
 
-	if (!option.gcHighlighting)
+	if (!PsiOptions::instance()->getOption("options.ui.muc.use-highlighting").toBool())
 		alert=false;
 
 	QDateTime time = QDateTime::currentDateTime();
 	if(!ts.isNull())
 		time = ts;
 
+	updateLastMsgTime(time);
 	QString timestr = ui_.log->formatTimeStamp(time);
 	ui_.log->appendText(QString("<font color=\"#00A000\">[%1]").arg(timestr) + QString(" *** %1</font>").arg(Qt::escape(str)));
 
@@ -1248,23 +1245,26 @@ QString GCMainDlg::getNickColor(QString nick)
 		}
 		sender=nicks[nick];
 	}
-
-	if(!option.gcNickColoring || option.gcNickColors.empty()) {
+	
+	QStringList nickColors = PsiOptions::instance()->getOption("options.ui.look.colors.muc.nick-colors").toStringList();
+	
+	if(!PsiOptions::instance()->getOption("options.ui.muc.use-nick-coloring").toBool() || nickColors.empty()) {
 		return "#000000";
 	}
-	else if(sender == -1 || option.gcNickColors.size() == 1) {
-		return option.gcNickColors[option.gcNickColors.size()-1];
+	else if(sender == -1 || nickColors.size() == 1) {
+		return nickColors[nickColors.size()-1];
 	}
 	else {
-		int n = sender % (option.gcNickColors.size()-1);
-		return option.gcNickColors[n];
+		int n = sender % (nickColors.size()-1);
+		return nickColors[n];
 	}
 }
 
 void GCMainDlg::appendMessage(const Message &m, bool alert)
 {
+	updateLastMsgTime(m.timeStamp());
 	//QString who, color;
-	if (!option.gcHighlighting)
+	if (!PsiOptions::instance()->getOption("options.ui.muc.use-highlighting").toBool())
 		alert=false;
 	QString who, textcolor, nickcolor,alerttagso,alerttagsc;
 
@@ -1301,7 +1301,7 @@ void GCMainDlg::appendMessage(const Message &m, bool alert)
 
 	txt = TextUtil::linkify(txt);
 
-	if(option.useEmoticons)
+	if(PsiOptions::instance()->getOption("options.ui.emoticons.use-emoticons").toBool())
 		txt = TextUtil::emoticonify(txt);
 	if( PsiOptions::instance()->getOption("options.ui.chat.legacy-formatting").toBool() )
 		txt = TextUtil::legacyFormat(txt);
@@ -1311,7 +1311,7 @@ void GCMainDlg::appendMessage(const Message &m, bool alert)
 		ui_.log->appendText(QString("<font color=\"%1\">").arg(nickcolor) + QString("[%1]").arg(timestr) + QString(" *%1 ").arg(Qt::escape(who)) + alerttagso + txt + alerttagsc + "</font>");
 	}
 	else {
-		if(option.chatSays) {
+		if(PsiOptions::instance()->getOption("options.ui.chat.use-chat-says-style").toBool()) {
 			//ui_.log->append(QString("<font color=\"%1\">").arg(color) + QString("[%1] ").arg(timestr) + QString("%1 says:").arg(Qt::escape(who)) + "</font><br>" + txt);
 			ui_.log->appendText(QString("<font color=\"%1\">").arg(nickcolor) + QString("[%1] ").arg(timestr) + QString("%1 says:").arg(Qt::escape(who)) + "</font><br>" + QString("<font color=\"%1\">").arg(textcolor) + alerttagso + txt + alerttagsc + "</font>");
 		}
@@ -1326,9 +1326,9 @@ void GCMainDlg::appendMessage(const Message &m, bool alert)
 		d->deferredScroll();
 
 	// if we're not active, notify the user by changing the title
-	if(!isActiveWindow()) {
+	if(!isActiveTab()) {
 		++d->pending;
-		updateCaption();
+		invalidateTab();
 	}
 
 	//if someone directed their comments to us, notify the user
@@ -1345,71 +1345,24 @@ void GCMainDlg::appendMessage(const Message &m, bool alert)
 
 void GCMainDlg::doAlert()
 {
-	if(!isActiveWindow())
+	if(!isActiveTab())
 		if (PsiOptions::instance()->getOption("options.ui.flash-windows").toBool())
 			doFlash(true);
 }
 
-void GCMainDlg::updateCaption()
+QString GCMainDlg::desiredCaption() const
 {
 	QString cap = "";
 
-	if(d->pending > 0) {
+	if (d->pending > 0) {
 		cap += "* ";
-		if(d->pending > 1)
+		if (d->pending > 1) {
 			cap += QString("[%1] ").arg(d->pending);
-	}
-	cap += d->jid.full();
-
-	// if taskbar flash, then we need to erase first and redo
-#ifdef Q_WS_WIN
-	bool on = false;
-	if(d->flashTimer)
-		on = d->flashCount & 1;
-	if(on)
-		FlashWindow(winId(), true);
-#endif
-	setWindowTitle(cap);
-#ifdef Q_WS_WIN
-	if(on)
-		FlashWindow(winId(), true);
-#endif
-}
-
-#ifdef Q_WS_WIN
-void GCMainDlg::doFlash(bool yes)
-{
-	if(yes) {
-		if(d->flashTimer)
-			return;
-		d->flashTimer = new QTimer(this);
-		connect(d->flashTimer, SIGNAL(timeout()), SLOT(flashAnimate()));
-		d->flashCount = 0;
-		flashAnimate(); // kick the first one immediately
-		d->flashTimer->start(500);
-	}
-	else {
-		if(d->flashTimer) {
-			delete d->flashTimer;
-			d->flashTimer = 0;
-			FlashWindow(winId(), false);
 		}
 	}
-}
-#else
-void GCMainDlg::doFlash(bool)
-{
-}
-#endif
+	cap += jid().full();
 
-void GCMainDlg::flashAnimate()
-{
-#ifdef Q_WS_WIN
-	FlashWindow(winId(), true);
-	++d->flashCount;
-	if(d->flashCount == 5)
-		d->flashTimer->stop();
-#endif
+	return cap;
 }
 
 void GCMainDlg::setLooks()
@@ -1419,11 +1372,11 @@ void GCMainDlg::setLooks()
 
 	// update the fonts
 	QFont f;
-	f.fromString(option.font[fChat]);
+	f.fromString(PsiOptions::instance()->getOption("options.ui.look.font.chat").toString());
 	ui_.log->setFont(f);
 	ui_.mle->chatEdit()->setFont(f);
 
-	f.fromString(option.font[fRoster]);
+	f.fromString(PsiOptions::instance()->getOption("options.ui.look.font.contactlist").toString());
 	ui_.lv_users->Q3ListView::setFont(f);
 
 	if (PsiOptions::instance()->getOption("options.ui.chat.central-toolbar").toBool()) {
@@ -1433,7 +1386,7 @@ void GCMainDlg::setLooks()
 	}
 	else {
 		ui_.toolbar->hide();
-		ui_.tb_emoticons->setVisible(option.useEmoticons);
+		ui_.tb_emoticons->setVisible(PsiOptions::instance()->getOption("options.ui.emoticons.use-emoticons").toBool());
 		ui_.tb_actions->show();
 	}
 
@@ -1461,14 +1414,14 @@ void GCMainDlg::optionsUpdate()
 void GCMainDlg::lv_action(const QString &nick, const Status &s, int x)
 {
 	if(x == 0) {
-		d->pa->invokeGCMessage(d->jid.withResource(nick));
+		account()->invokeGCMessage(jid().withResource(nick));
 	}
 	else if(x == 1) {
-		d->pa->invokeGCChat(d->jid.withResource(nick));
+		account()->invokeGCChat(jid().withResource(nick));
 	}
 	else if(x == 2) {
 		UserListItem u;
-		u.setJid(d->jid.withResource(nick));
+		u.setJid(jid().withResource(nick));
 		u.setName(nick);
 
 		// make a resource so the contact appears online
@@ -1481,10 +1434,10 @@ void GCMainDlg::lv_action(const QString &nick, const Status &s, int x)
 		w->show();
 	}
 	else if(x == 3) {
-		d->pa->invokeGCInfo(d->jid.withResource(nick));
+		account()->invokeGCInfo(jid().withResource(nick));
 	}
 	else if(x == 4) {
-		d->pa->invokeGCFile(d->jid.withResource(nick));
+		account()->invokeGCFile(jid().withResource(nick));
 	}
 	else if(x == 10) {
 		d->mucManager->kick(nick);
@@ -1558,6 +1511,16 @@ void GCMainDlg::chatEditCreated()
 	ui_.mle->chatEdit()->installEventFilter(d);
 }
 
+TabbableWidget::State GCMainDlg::state() const
+{
+	return TabbableWidget::StateNone;
+}
+
+int GCMainDlg::unreadMessageCount() const
+{
+	return d->pending;
+}
+
 //----------------------------------------------------------------------------
 // GCFindDlg
 //----------------------------------------------------------------------------
