@@ -32,10 +32,13 @@
 #include <qmessagebox.h>
 #include <QtCrypto>
 #include <QTranslator>
+#include <QDir>
+
 #include <stdlib.h>
 #include <time.h>
-#include "profiles.h"
 #include "profiledlg.h"
+#include "activeprofiles.h"
+#include "psioptions.h"
 
 #include "eventdlg.h"
 #include "psicon.h"
@@ -121,6 +124,13 @@ PsiMain::PsiMain(QObject *par)
 			QTimer::singleShot(0, this, SLOT(bail()));
 		}
 		else {
+			PsiOptions o;
+			if (!o.newProfile()) {
+				qWarning("ERROR: Failed to new profile default options");
+			}
+
+			o.save(pathToProfile("default") + "/options.xml");
+
 			lastProfile = activeProfile = "default";
 			autoOpen = true;
 			QTimer::singleShot(0, this, SLOT(sessionStart()));
@@ -182,10 +192,30 @@ void PsiMain::chooseProfile()
 			continue;
 		}
 		else {
-			if(r == QDialog::Accepted)
-				str = w->cb_profile->currentText();
+			bool again = false;
 			autoOpen = w->ck_auto->isChecked();
+			if(r == QDialog::Accepted) {
+				str = w->cb_profile->currentText();
+				again = !ActiveProfiles::instance()->setThisProfile(str);
+				if (again) {
+					QMessageBox mb(QMessageBox::Question,
+						CAP(tr("Profile already in use")),
+						QString(tr("The \"%1\" profile is already in use.\nWould you like to activate that session now?")).arg(str),
+						QMessageBox::Cancel);
+					QPushButton *activate = mb.addButton(tr("Activate"), QMessageBox::AcceptRole);
+					mb.exec();
+					if (mb.clickedButton() == activate) {
+						ActiveProfiles::instance()->raiseOther(str, true);
+						quit();
+						return;
+					}
+				}
+			}
 			delete w;
+			if (again) {
+				str = "";
+				continue;
+			}
 			break;
 		}
 	}
@@ -204,7 +234,15 @@ void PsiMain::chooseProfile()
 
 void PsiMain::sessionStart()
 {
+	// make sure we have clean PsiOptions
+	PsiOptions::reset();
 	// get a PsiCon
+	if (!ActiveProfiles::instance()->setThisProfile(activeProfile)) { // already running
+		if (!ActiveProfiles::instance()->raiseOther(activeProfile, true))
+			QMessageBox::critical(0, tr("Error"), tr("Cannot open this profile - it is already running, but not responding"));
+		quit();
+		return;
+	}
 	pcon = new PsiCon();
 	if(!pcon->init()) {
 		delete pcon;
@@ -239,10 +277,17 @@ int main(int argc, char *argv[])
 	// it must be initialized first in order for ApplicationInfo::resourcesDir() to work
 	QCA::Initializer init;
 	PsiApplication app(argc, argv);
-	QApplication::addLibraryPath(ApplicationInfo::homeDir());
 	QApplication::addLibraryPath(ApplicationInfo::resourcesDir());
+	QApplication::addLibraryPath(ApplicationInfo::homeDir());
 	QApplication::setQuitOnLastWindowClosed(false);
 
+#ifdef Q_WS_MAC
+	QDir dir(QApplication::applicationDirPath());
+	dir.cdUp();
+	dir.cd("Plugins");
+	QApplication::addLibraryPath(dir.absolutePath());
+#endif
+
 	// Initialize QCA
 	QCA::setProperty("pgp-always-trust", true);
 	QCA::KeyStoreManager keystoremgr;
