@@ -20,7 +20,6 @@
 
 #include "mainwin.h"
 
-#include <QDesktopServices>
 #include <qmessagebox.h>
 #include <qicon.h>
 #include <qapplication.h>
@@ -36,6 +35,7 @@
 #include <QVBoxLayout>
 #include <QMenu>
 #include <QMenuItem>
+#include <QtAlgorithms>
 
 #ifdef Q_WS_WIN
 #include <windows.h>
@@ -58,25 +58,13 @@
 #include "tipdlg.h"
 #include "mucjoindlg.h"
 #include "psicontactlist.h"
+#include "desktoputil.h"
 
 #include "mainwin_p.h"
 
 using namespace XMPP;
 
-// deletes submenus in a popupmenu
-/*void qpopupmenuclear(QMenu *p)
-{
-	while(p->count()) {
-		QMenuItem *item = p->findItem(p->idAt(0));
-		QMenu *popup = item->menu();
-		p->removeItemAt(0);
 
-		if(popup)
-			delete popup;
-	}
-}*/
-
-	
 //----------------------------------------------------------------------------
 // MainWin::Private
 //----------------------------------------------------------------------------
@@ -87,40 +75,43 @@ public:
 	Private(PsiCon *, MainWin *);
 	~Private();
 
-	QVBoxLayout *vb_main;
+	QVBoxLayout* vb_main;
 	bool onTop, asTool;
-	QMenu *mainMenu, *statusMenu, *optionsMenu, *toolsMenu;
+	QMenu* mainMenu, *statusMenu, *optionsMenu, *toolsMenu;
 	int sbState;
 	QString nickname;
-	PsiTrayIcon *tray;
-	QMenu *trayMenu;
+	PsiTrayIcon* tray;
+	QMenu* trayMenu;
 	QString statusTip;
 
-	PopupAction *optionsButton, *statusButton;
-	IconActionGroup *statusGroup;
-	EventNotifierAction *eventNotifier;
-	PsiCon *psi;
-	MainWin *mainWin;
+	PopupAction* optionsButton, *statusButton;
+	IconActionGroup* statusGroup;
+	EventNotifierAction* eventNotifier;
+	PsiCon* psi;
+	MainWin* mainWin;
 
-	QSignalMapper *statusMapper;
+	QLineEdit* searchText;
+	QToolButton* searchPb;
+	QWidget* searchWidget;
 
-	PsiIcon *nextAnim;
+	QSignalMapper* statusMapper;
+
+	PsiIcon* nextAnim;
 	int nextAmount;
 
 	QMap<QAction *, int> statusActions;
 
 	int lastStatus;
 	bool old_trayicon;
+	bool filterActive, prefilterShowOffline, prefilterShowAway;
 
 	void registerActions();
-	IconAction *getAction( QString name );
-	void updateMenu(QStringList actions, QMenu *menu);
+	IconAction* getAction( QString name );
+	void updateMenu(QStringList actions, QMenu* menu);
 };
 
-MainWin::Private::Private(PsiCon *_psi, MainWin *_mainWin)
+MainWin::Private::Private(PsiCon* _psi, MainWin* _mainWin) : psi(_psi), mainWin(_mainWin)
 {
-	psi = _psi;
-	mainWin = _mainWin;
 
 	statusGroup   = (IconActionGroup *)getAction("status_all");
 	eventNotifier = (EventNotifierAction *)getAction("event_notifier");
@@ -130,6 +121,10 @@ MainWin::Private::Private(PsiCon *_psi, MainWin *_mainWin)
 
 	statusMapper = new QSignalMapper(mainWin, "statusMapper");
 	mainWin->connect(statusMapper, SIGNAL(mapped(int)), mainWin, SLOT(activatedStatusAction(int)));
+  
+	filterActive = false;
+	prefilterShowOffline = false;
+	prefilterShowAway = false;  
 }
 
 MainWin::Private::~Private()
@@ -139,7 +134,7 @@ MainWin::Private::~Private()
 void MainWin::Private::registerActions()
 {
 	struct {
-		const char *name;
+		const char* name;
 		int id;
 	} statuslist[] = {
 		{ "status_chat",      STATUS_CHAT      },
@@ -155,7 +150,7 @@ void MainWin::Private::registerActions()
 	int i;
 	QString aName;
 	for ( i = 0; !(aName = QString(statuslist[i].name)).isEmpty(); i++ ) {
-		IconAction *action = getAction( aName );
+		IconAction* action = getAction( aName );
 		connect (action, SIGNAL(activated()), statusMapper, SLOT(map()));
 
 		statusMapper->setMapping(action, statuslist[i].id);
@@ -168,31 +163,33 @@ void MainWin::Private::registerActions()
 	QStringList names = actions.actions();
 	QStringList::Iterator it = names.begin();
 	for ( ; it != names.end(); ++it ) {
-		IconAction *action = actions.action( *it );
-		if ( action )
+		IconAction* action = actions.action( *it );
+		if ( action ) {
 			mainWin->registerAction( action );
+		}
 	}
 }
 
-IconAction *MainWin::Private::getAction( QString name )
+IconAction* MainWin::Private::getAction( QString name )
 {
 	PsiActionList::ActionsType type = PsiActionList::ActionsType( PsiActionList::Actions_MainWin | PsiActionList::Actions_Common );
 	ActionList actions = psi->actionList()->suitableActions( type );
-	IconAction *action = actions.action( name );
+	IconAction* action = actions.action( name );
 
-	if ( !action )
+	if ( !action ) {
 		qWarning("MainWin::Private::getAction(): action %s not found!", name.latin1());
+	}
 	//else
 	//	mainWin->registerAction( action );
 
 	return action;
 }
 
-void MainWin::Private::updateMenu(QStringList actions, QMenu *menu)
+void MainWin::Private::updateMenu(QStringList actions, QMenu* menu)
 {
 	menu->clear();
 
-	IconAction *action;
+	IconAction* action;
 	foreach (QString name, actions) {
 		// workind around Qt/X11 bug, which displays
 		// actions's text and the separator bar in Qt 4.1.1
@@ -200,9 +197,18 @@ void MainWin::Private::updateMenu(QStringList actions, QMenu *menu)
 			menu->insertSeparator();
 			continue;
 		}
-		
-		if ( (action = getAction(name)) )
+
+		if ( name == "diagnostics" ) {
+			QMenu* diagMenu = new QMenu(mainWin);
+			menu->insertItem(tr("Diagnostics"), diagMenu);
+			getAction("help_diag_qcaplugin")->addTo(diagMenu);
+			getAction("help_diag_qcakeystore")->addTo(diagMenu);
+			continue;
+		}
+
+		if ( (action = getAction(name)) ) {
 			action->addTo(menu);
+		}
 	}
 }
 
@@ -222,13 +228,12 @@ void MainWin::Private::updateMenu(QStringList actions, QMenu *menu)
 #define TOOLW_FLAGS ((Qt::WFlags) 0)
 #endif
 
-MainWin::MainWin(bool _onTop, bool _asTool, PsiCon *psi, const char *name)
-:AdvancedWidget<Q3MainWindow>(0, (_onTop ? Qt::WStyle_StaysOnTop : Qt::Widget) | (_asTool ? (Qt::WStyle_Tool |TOOLW_FLAGS) : Qt::Widget))
-//: Q3MainWindow(0,name,(_onTop ? Qt::WStyle_StaysOnTop : Qt::Widget) | (_asTool ? (Qt::WStyle_Tool |TOOLW_FLAGS) : Qt::Widget))
+MainWin::MainWin(bool _onTop, bool _asTool, PsiCon* psi, const char* name)
+:AdvancedWidget<QMainWindow>(0, (_onTop ? Qt::WStyle_StaysOnTop : Qt::Widget) | (_asTool ? (Qt::WStyle_Tool |TOOLW_FLAGS) : Qt::Widget))
 {
 	setObjectName(name);
 	setAttribute(Qt::WA_AlwaysShowToolTips);
-  	if ( option.brushedMetal ) {
+  	if ( PsiOptions::instance()->getOption("options.ui.mac.use-brushed-metal-windows").toBool() ) {
 		setAttribute(Qt::WA_MacMetalStyle);
 	}
 	d = new Private(psi, this);
@@ -256,7 +261,7 @@ MainWin::MainWin(bool _onTop, bool _asTool, PsiCon *psi, const char *name)
 	d->old_trayicon = PsiOptions::instance()->getOption("options.ui.systemtray.use-old").toBool();
 #endif
 
-	QWidget *center = new QWidget (this, "Central widget");
+	QWidget* center = new QWidget (this, "Central widget");
 	setCentralWidget ( center );
 
 	d->vb_main = new QVBoxLayout(center);
@@ -270,6 +275,22 @@ MainWin::MainWin(bool _onTop, bool _asTool, PsiCon *psi, const char *name)
 	d->vb_main->setMargin(layoutMargin);
 	d->vb_main->setSpacing(layoutMargin);
 
+	// create search bar
+	d->searchWidget = new QWidget(centralWidget());
+	d->vb_main->addWidget(d->searchWidget);
+	QHBoxLayout* searchLayout = new QHBoxLayout(d->searchWidget);
+	searchLayout->setMargin(0);
+	searchLayout->setSpacing(0);
+	d->searchText = new QLineEdit(d->searchWidget);
+	connect(d->searchText,SIGNAL(textEdited(const QString&)),SLOT(searchTextEntered(const QString&)));
+	searchLayout->addWidget(d->searchText);
+	d->searchPb = new QToolButton(d->searchWidget);
+	d->searchPb->setText("X");
+	connect(d->searchPb,SIGNAL(clicked()),SLOT(searchClearClicked()));
+	connect(cvlist, SIGNAL(searchInput(const QString&)), SLOT(searchTextStarted(const QString&)));
+	searchLayout->addWidget(d->searchPb);
+	d->searchWidget->setVisible(false);
+	//add contact view
 	d->vb_main->addWidget(cvlist);
 
 #ifdef Q_WS_MAC
@@ -311,10 +332,10 @@ MainWin::MainWin(bool _onTop, bool _asTool, PsiCon *psi, const char *name)
 
 	// Mac-only menus
 #ifdef Q_WS_MAC
-	QMenu *mainMenu = new QMenu(this);
+	QMenu* mainMenu = new QMenu(this);
 	mainMenuBar()->insertItem(tr("Menu"), mainMenu);
-	mainMenu->insertItem(tr("Preferences"), this, SIGNAL(doOptions()));
-	mainMenu->insertItem(tr("Quit"), this, SLOT(try2tryCloseProgram()));
+	d->getAction("menu_options")->addTo(mainMenu);
+	d->getAction("menu_quit")->addTo(mainMenu);
 	d->getAction("help_about")->addTo(mainMenu);
 	d->getAction("help_about_qt")->addTo(mainMenu);
 
@@ -327,11 +348,12 @@ MainWin::MainWin(bool _onTop, bool _asTool, PsiCon *psi, const char *name)
 
 	mainMenuBar()->insertItem(tr("Status"), d->statusMenu);
 
-	QMenu *viewMenu = new QMenu(this);
+	QMenu* viewMenu = new QMenu(this);
 	mainMenuBar()->insertItem(tr("View"), viewMenu);
 	d->getAction("show_offline")->addTo(viewMenu);
-	if (PsiOptions::instance()->getOption("options.ui.menu.view.show-away").toBool())
+	if (PsiOptions::instance()->getOption("options.ui.menu.view.show-away").toBool()) {
 		d->getAction("show_away")->addTo(viewMenu);
+	}
 	d->getAction("show_hidden")->addTo(viewMenu);
 	d->getAction("show_agents")->addTo(viewMenu);
 	d->getAction("show_self")->addTo(viewMenu);
@@ -344,7 +366,7 @@ MainWin::MainWin(bool _onTop, bool _asTool, PsiCon *psi, const char *name)
 	mainMenuBar()->insertItem(tr("Tools"), d->toolsMenu);
 	connect(d->toolsMenu, SIGNAL(aboutToShow()), SLOT(buildToolsMenu()));
 
-	QMenu *helpMenu = new QMenu(this);
+	QMenu* helpMenu = new QMenu(this);
 	mainMenuBar()->insertItem(tr("Help"), helpMenu);
 	d->getAction("help_readme")->addTo (helpMenu);
 	d->getAction("help_tip")->addTo (helpMenu);
@@ -354,24 +376,28 @@ MainWin::MainWin(bool _onTop, bool _asTool, PsiCon *psi, const char *name)
 	d->getAction("help_online_home")->addTo (helpMenu);
 	d->getAction("help_psi_muc")->addTo (helpMenu);
 	d->getAction("help_report_bug")->addTo (helpMenu);
+	QMenu* diagMenu = new QMenu(this);
+	helpMenu->insertItem(tr("Diagnostics"), diagMenu);
+	d->getAction("help_diag_qcaplugin")->addTo (diagMenu);
+	d->getAction("help_diag_qcakeystore")->addTo (diagMenu);
 #else
-	if (option.hideMenubar) 
+	if (!PsiOptions::instance()->getOption("options.ui.contactlist.show-menubar").toBool())  {
 		mainMenuBar()->hide();
+	}
 	//else 
 	//	mainMenuBar()->show();
 #endif
 	d->optionsButton->setMenu( d->optionsMenu );
 	d->statusButton->setMenu( d->statusMenu );
-	
+
 	buildToolbars();
-	
-	setWindowOpacity(double(qMax(MINIMUM_OPACITY,PsiOptions::instance()->getOption("options.ui.contactlist.opacity").toInt()))/100);
+	// setUnifiedTitleAndToolBarOnMac(true);
 
 	connect(qApp, SIGNAL(dockActivated()), SLOT(dockActivated()));
 
-	connect(PsiOptions::instance(), SIGNAL(optionChanged(const QString&)), SLOT(optionsUpdate()));
+	connect(psi, SIGNAL(emitOptionsUpdate()), SLOT(optionsUpdate()));
+	optionsUpdate();
 }
-	
 
 MainWin::~MainWin()
 {
@@ -382,25 +408,21 @@ MainWin::~MainWin()
 		d->tray = 0;
 	}
 
-	//saveToolbarsPositions();
-	// need to find some workaround to case, when you're logging off. in that case
-	// toobars are all disabled, and when you start psi again you need to enable
-	// your toolbars
-
+	saveToolbarsState();
 	delete d;
 }
 
-void MainWin::registerAction( IconAction *action )
+void MainWin::registerAction( IconAction* action )
 {
 	char activated[] = SIGNAL( activated() );
 	char toggled[]   = SIGNAL( toggled(bool) );
 	char setChecked[]     = SLOT( setChecked(bool) );
 
 	struct {
-		const char *name;
-		const char *signal;
-		QObject *receiver;
-		const char *slot;
+		const char* name;
+		const char* signal;
+		QObject* receiver;
+		const char* slot;
 	} actionlist[] = {
 		{ "show_offline", toggled, cvlist, SLOT( setShowOffline(bool) ) },
 		{ "show_away",    toggled, cvlist, SLOT( setShowAway(bool) ) },
@@ -440,6 +462,8 @@ void MainWin::registerAction( IconAction *action )
 		{ "help_report_bug",  activated, this, SLOT( actBugReportActivated() ) },
 		{ "help_about",       activated, this, SLOT( actAboutActivated() ) },
 		{ "help_about_qt",    activated, this, SLOT( actAboutQtActivated() ) },
+		{ "help_diag_qcaplugin",   activated, this, SLOT( actDiagQCAPluginActivated() ) },
+		{ "help_diag_qcakeystore", activated, this, SLOT( actDiagQCAKeyStoreActivated() ) },
 
 		{ "", 0, 0, 0 }
 	};
@@ -450,34 +474,37 @@ void MainWin::registerAction( IconAction *action )
 		if ( aName == action->name() ) {
 #ifdef USE_PEP
 			// Check before connecting, otherwise we get a loop
-			if ( aName == "publish_tune")
+			if ( aName == "publish_tune") {
 				action->setChecked( PsiOptions::instance()->getOption("options.extended-presence.tune.publish").toBool() );
+			}
 #endif
 
 			disconnect( action, actionlist[i].signal, actionlist[i].receiver, actionlist[i].slot ); // for safety
 			connect( action, actionlist[i].signal, actionlist[i].receiver, actionlist[i].slot );
 
 			// special cases
-			if ( aName == "menu_play_sounds" )
-				action->setChecked( useSound );
+			if ( aName == "menu_play_sounds" ) {
+				action->setChecked(PsiOptions::instance()->getOption("options.ui.notifications.sounds.enable").toBool());
+			}
 			//else if ( aName == "foobar" )
 			//	;
 		}
 	}
 
 	struct {
-		const char *name;
-		QObject *sender;
-		const char *signal;
-		const char *slot;
+		const char* name;
+		QObject* sender;
+		const char* signal;
+		const char* slot;
+		bool checked;
 	} reverseactionlist[] = {
-		{ "show_away",    cvlist, SIGNAL( showAway(bool) ), setChecked },
-		{ "show_hidden",  cvlist, SIGNAL( showHidden(bool) ), setChecked },
-		{ "show_offline", cvlist, SIGNAL( showOffline(bool) ), setChecked },
-		{ "show_self",    cvlist, SIGNAL( showSelf(bool) ), setChecked },
-		{ "show_agents",  cvlist, SIGNAL( showAgents(bool) ), setChecked },
-		{ "show_statusmsg", cvlist, SIGNAL( showStatusMsg(bool) ), setChecked },
-		{ "", 0, 0, 0 }
+		{ "show_away",    cvlist, SIGNAL( showAway(bool) ), setChecked, cvlist->isShowAway()},
+		{ "show_hidden",  cvlist, SIGNAL( showHidden(bool) ), setChecked, cvlist->isShowHidden()},
+		{ "show_offline", cvlist, SIGNAL( showOffline(bool) ), setChecked, cvlist->isShowOffline()},
+		{ "show_self",    cvlist, SIGNAL( showSelf(bool) ), setChecked, cvlist->isShowSelf()},
+		{ "show_agents",  cvlist, SIGNAL( showAgents(bool) ), setChecked, cvlist->isShowAgents()},
+		{ "show_statusmsg", cvlist, SIGNAL( showStatusMsg(bool) ), setChecked, cvlist->isShowStatusMsg()},
+		{ "", 0, 0, 0, false }
 	};
 
 	for ( i = 0; !(aName = QString(reverseactionlist[i].name)).isEmpty(); i++ ) {
@@ -489,29 +516,32 @@ void MainWin::registerAction( IconAction *action )
 				action->setChecked( PsiOptions::instance()->getOption("options.ui.contactlist.status-messages.show").toBool() );
 			}
 			else
-				action->setChecked( true );
+				action->setChecked( reverseactionlist[i].checked );
 		}
 	}
 }
 
-PsiCon *MainWin::psiCon() const
+PsiCon* MainWin::psiCon() const
 {
 	return d->psi;
 }
 
 void MainWin::setWindowOpts(bool _onTop, bool _asTool)
 {
-	if(_onTop == d->onTop && _asTool == d->asTool)
+	if(_onTop == d->onTop && _asTool == d->asTool) {
 		return;
+	}
 
 	d->onTop = _onTop;
 	d->asTool = _asTool;
 
 	Qt::WFlags flags = 0;
-	if(d->onTop)
+	if(d->onTop) {
 		flags |= Qt::WStyle_StaysOnTop;
-	if(d->asTool)
+	}
+	if(d->asTool) {
 		flags |= Qt::WStyle_Tool | TOOLW_FLAGS;
+	}
 
 	QPoint p = pos();
 	reparent(parentWidget(), flags, p, FALSE);
@@ -521,44 +551,45 @@ void MainWin::setWindowOpts(bool _onTop, bool _asTool)
 
 void MainWin::setUseDock(bool use)
 {
-	if(use == false || d->tray) {
-		if(d->tray) {
-			delete d->tray;
-			d->tray = 0;
-		}
-
-		if (use == false)
-			return;
-	}
-
-	if(d->tray)
+	if (use == (d->tray != 0)) {
 		return;
-
-	d->tray = new PsiTrayIcon("Psi", d->trayMenu, d->old_trayicon);
-	if (d->old_trayicon) {
-		connect(d->tray, SIGNAL(closed()), SLOT(dockActivated()));
-		connect(qApp, SIGNAL(trayOwnerDied()), SLOT(dockActivated()));
 	}
-	connect(d->tray, SIGNAL(clicked(const QPoint &, int)), SLOT(trayClicked(const QPoint &, int)));
-	connect(d->tray, SIGNAL(doubleClicked(const QPoint &)), SLOT(trayDoubleClicked()));
-	d->tray->setIcon( PsiIconset::instance()->statusPtr( STATUS_OFFLINE ));
-	d->tray->setToolTip(ApplicationInfo::name());
 
-	updateReadNext(d->nextAnim, d->nextAmount);
+	if (d->tray) {
+		delete d->tray;
+		d->tray = 0;
+	}
 
-	d->tray->show();
+	Q_ASSERT(!d->tray);
+	if (use) {
+		d->tray = new PsiTrayIcon("Psi", d->trayMenu, d->old_trayicon);
+		if (d->old_trayicon) {
+			connect(d->tray, SIGNAL(closed()), SLOT(dockActivated()));
+			connect(qApp, SIGNAL(trayOwnerDied()), SLOT(dockActivated()));
+		}
+		connect(d->tray, SIGNAL(clicked(const QPoint &, int)), SLOT(trayClicked(const QPoint &, int)));
+		connect(d->tray, SIGNAL(doubleClicked(const QPoint &)), SLOT(trayDoubleClicked()));
+		d->tray->setIcon(PsiIconset::instance()->statusPtr(STATUS_OFFLINE));
+		d->tray->setToolTip(ApplicationInfo::name());
+
+		updateReadNext(d->nextAnim, d->nextAmount);
+
+		d->tray->show();
+	}
 }
 
 void MainWin::buildStatusMenu()
 {
 	d->statusMenu->clear();
 	d->getAction("status_online")->addTo(d->statusMenu);
-	if (PsiOptions::instance()->getOption("options.ui.menu.status.chat").toBool())
+	if (PsiOptions::instance()->getOption("options.ui.menu.status.chat").toBool()) {
 		d->getAction("status_chat")->addTo(d->statusMenu);
+	}
 	d->statusMenu->insertSeparator();
 	d->getAction("status_away")->addTo(d->statusMenu);
-	if (PsiOptions::instance()->getOption("options.ui.menu.status.xa").toBool())
+	if (PsiOptions::instance()->getOption("options.ui.menu.status.xa").toBool()) {
 		d->getAction("status_xa")->addTo(d->statusMenu);
+	}
 	d->getAction("status_dnd")->addTo(d->statusMenu);
 	if (PsiOptions::instance()->getOption("options.ui.menu.status.invisible").toBool()) {
 		d->statusMenu->insertSeparator();
@@ -576,7 +607,7 @@ void MainWin::activatedStatusAction(int id)
 {
 	QObjectList l = d->statusGroup->queryList( "IconAction" );
 	for (QObjectList::Iterator it = l.begin() ; it != l.end(); ++it) {
-		IconAction *action = (IconAction *)(*it);
+		IconAction* action = (IconAction *)(*it);
 		action->setChecked ( d->statusActions[action] == id );
 	}
 
@@ -592,46 +623,50 @@ QMenuBar* MainWin::mainMenuBar() const
 #endif
 }
 
-void MainWin::buildToolbars()
+const QString toolbarsStateOptionPath = "options.ui.contactlist.toolbars-state";
+
+void MainWin::saveToolbarsState()
 {
-	while ( option.toolbars["mainWin"].count() < toolbars.count() && toolbars.count() ) {
-		PsiToolBar *tb = toolbars.last();
-		toolbars.removeLast();
-		delete tb;
-	}
-
-	for (int i = 0; i < option.toolbars["mainWin"].count(); i++) {
-		PsiToolBar *tb = 0;
-		if ( i < toolbars.count() )
-			tb = toolbars.at(i);
-
-		Options::ToolbarPrefs &tbPref = option.toolbars["mainWin"][i];
-		if ( tb && !tbPref.dirty )
-			continue;
-
-		if ( tb )
-			delete tb;
-
-		tb = new PsiToolBar(tbPref.name, this, d->psi);
-		moveDockWindow ( tb, tbPref.dock, tbPref.nl, tbPref.index, tbPref. extraOffset );
-
-		tb->setGroup( "mainWin", i );
-		tb->setType( PsiActionList::Actions_MainWin );
-		//connect( tb, SIGNAL( registerAction( IconAction * ) ), SLOT( registerAction( IconAction * ) ) );
-		tb->initialize( tbPref, false );
-
-		if ( i < toolbars.count() )
-			toolbars.removeAt(i);
-		toolbars.insert(i, tb);
-	}
+	PsiOptions::instance()->setOption(toolbarsStateOptionPath, saveState());
 }
 
-void MainWin::saveToolbarsPositions()
+void MainWin::loadToolbarsState()
 {
-	for (int i = 0; i < toolbars.count(); i++) {
-		Options::ToolbarPrefs &tbPref = option.toolbars["mainWin"][i];
-		getLocation ( toolbars.at(i), tbPref.dock, tbPref.index, tbPref.nl, tbPref.extraOffset );
-		tbPref.on = toolbars.at(i)->isVisible();
+	restoreState(PsiOptions::instance()->getOption(toolbarsStateOptionPath).toByteArray());
+}
+
+void MainWin::buildToolbars()
+{
+	setUpdatesEnabled(false);
+	if (toolbars_.count() > 0) {
+		saveToolbarsState();
+	}
+
+	qDeleteAll(toolbars_);
+	toolbars_.clear();
+
+	foreach(QString base, PsiOptions::instance()->getChildOptionNames("options.ui.contactlist.toolbars", true, true)) {
+		PsiToolBar* tb = new PsiToolBar(base, this, d->psi->actionList());
+		tb->initialize();
+		connect(tb, SIGNAL(customize()), d->psi, SLOT(doToolbars()));
+		toolbars_ << tb;
+	}
+
+	loadToolbarsState();
+
+	// loadToolbarsState also restores correct toolbar visibility,
+	// we might want to override that
+	foreach(PsiToolBar* tb, toolbars_) {
+		tb->updateVisibility();
+	}
+
+	// d->eventNotifier->updateVisibility();
+	setUpdatesEnabled(true);
+
+	// in case we have floating toolbars, they have inherited the 'no updates enabled'
+	// state. now we need to explicitly re-enable updates.
+	foreach(PsiToolBar* tb, toolbars_) {
+		tb->setUpdatesEnabled(true);
 	}
 }
 
@@ -643,7 +678,7 @@ bool MainWin::showDockMenu(const QPoint &)
 void MainWin::buildOptionsMenu()
 {
 	// help menu
-	QMenu *helpMenu = new QMenu(d->optionsMenu);
+	QMenu* helpMenu = new QMenu(d->optionsMenu);
 
 	QStringList actions;
 	actions << "help_readme"
@@ -654,6 +689,7 @@ void MainWin::buildOptionsMenu()
 	        << "help_online_home"
 	        << "help_psi_muc"
 	        << "help_report_bug"
+	        << "diagnostics"
 	        << "separator"
 	        << "help_about"
 	        << "help_about_qt";
@@ -673,14 +709,16 @@ void MainWin::buildMainMenu()
 	// main menu
 	QStringList actions;
 	actions << "menu_add_contact";
-	if (PsiOptions::instance()->getOption("options.ui.message.enabled").toBool())
+	if (PsiOptions::instance()->getOption("options.ui.message.enabled").toBool()) {
 		actions << "menu_new_message";
+	}
 	actions << "menu_disco"
 	        << "menu_join_groupchat"
 	        << "separator"
 	        << "menu_account_setup";
-	if (PsiOptions::instance()->getOption("options.ui.menu.main.change-profile").toBool())
+	if (PsiOptions::instance()->getOption("options.ui.menu.main.change-profile").toBool()) {
 	        actions << "menu_change_profile";
+	}
 	actions << "menu_play_sounds";
 
 	d->updateMenu(actions, d->mainMenu);
@@ -696,20 +734,22 @@ void MainWin::buildToolsMenu()
 	d->updateMenu(actions, d->toolsMenu);
 }
 	
-void MainWin::buildGeneralMenu(QMenu *menu)
+void MainWin::buildGeneralMenu(QMenu* menu)
 {
 	// options menu
 	QStringList actions;
 	actions << "menu_add_contact";
-	if (PsiOptions::instance()->getOption("options.ui.message.enabled").toBool())
+	if (PsiOptions::instance()->getOption("options.ui.message.enabled").toBool()) {
 		actions << "menu_new_message";
+	}
 	actions << "menu_disco"
 	        << "menu_join_groupchat"
 	        << "menu_account_setup"
 	        << "menu_options"
 	        << "menu_file_transfer";
-	if (PsiOptions::instance()->getOption("options.ui.menu.main.change-profile").toBool())
+	if (PsiOptions::instance()->getOption("options.ui.menu.main.change-profile").toBool()) {
 	        actions << "menu_change_profile";
+	}
 	actions << "menu_play_sounds";
 
 	d->updateMenu(actions, menu);
@@ -717,46 +757,44 @@ void MainWin::buildGeneralMenu(QMenu *menu)
 
 void MainWin::actReadmeActivated ()
 {
-	ShowTextDlg *w = new ShowTextDlg(":/README");
+	ShowTextDlg* w = new ShowTextDlg(":/README");
 	w->setWindowTitle(CAP(tr("ReadMe")));
 	w->show();
 }
 
 void MainWin::actOnlineHelpActivated ()
 {
-	QDesktopServices::openUrl(QUrl("http://psi-im.org/wiki/User_Guide"));
+	DesktopUtil::openUrl("http://psi-im.org/wiki/User_Guide");
 }
 
 void MainWin::actOnlineWikiActivated ()
 {
-	QDesktopServices::openUrl(QUrl("http://psi-im.org/wiki"));
+	DesktopUtil::openUrl("http://psi-im.org/wiki");
 }
 
 void MainWin::actOnlineHomeActivated ()
 {
-	QDesktopServices::openUrl(QUrl("http://psi-im.org"));
+	DesktopUtil::openUrl("http://psi-im.org");
 }
 
 void MainWin::actJoinPsiMUCActivated()
 {
-	PsiAccount *account = d->psi->contactList()->defaultAccount();
-	if(!account)
+	PsiAccount* account = d->psi->contactList()->defaultAccount();
+	if(!account) {
 		return;
+	}
 
-	MUCJoinDlg *w = new MUCJoinDlg(d->psi, account);
-	w->le_host->setText("conference.psi-im.org");
-	w->le_room->setText("psi");
-	w->show();
+	account->actionJoin("psi@conference.psi-im.org");
 }
 
 void MainWin::actBugReportActivated ()
 {
-	QDesktopServices::openUrl(QUrl("http://psi-im.org/forum/forum/2"));
+	DesktopUtil::openUrl("http://psi-im.org/forum/forum/2");
 }
 
 void MainWin::actAboutActivated ()
 {
-	AboutDlg *about = new AboutDlg();
+	AboutDlg* about = new AboutDlg();
 	about->show();
 }
 
@@ -770,9 +808,27 @@ void MainWin::actAboutQtActivated ()
 	QMessageBox::aboutQt(this);
 }
 
+void MainWin::actDiagQCAPluginActivated()
+{
+	QString dtext = QCA::pluginDiagnosticText();
+	ShowTextDlg* w = new ShowTextDlg(dtext, true, false, this);
+	w->setWindowTitle(CAP(tr("Security Plugins Diagnostic Text")));
+	w->resize(560, 240);
+	w->show();
+}
+
+void MainWin::actDiagQCAKeyStoreActivated()
+{
+	QString dtext = QCA::KeyStoreManager::diagnosticText();
+	ShowTextDlg* w = new ShowTextDlg(dtext, true, false, this);
+	w->setWindowTitle(CAP(tr("Key Storage Diagnostic Text")));
+	w->resize(560, 240);
+	w->show();
+}
+
 void MainWin::actPlaySoundsActivated (bool state)
 {
-	useSound = state;
+	PsiOptions::instance()->setOption("options.ui.notifications.sounds.enable", state);
 }
 
 void MainWin::actPublishTuneActivated (bool state)
@@ -780,14 +836,17 @@ void MainWin::actPublishTuneActivated (bool state)
 	PsiOptions::instance()->setOption("options.extended-presence.tune.publish",state);
 }
 
-void MainWin::activatedAccOption(PsiAccount *pa, int x)
+void MainWin::activatedAccOption(PsiAccount* pa, int x)
 {
-	if(x == 0)
+	if(x == 0) {
 		pa->openAddUserDlg();
-	else if(x == 2)
+	}
+	else if(x == 2) {
 		pa->showXmlConsole();
-	else if(x == 3)
+	}
+	else if(x == 3) {
 		pa->doDisco();
+	}
 }
 
 void MainWin::buildTrayMenu()
@@ -800,10 +859,12 @@ void MainWin::buildTrayMenu()
 		d->trayMenu->insertSeparator();
 	}
 
-	if(isHidden())
+	if(isHidden()) {
 		d->trayMenu->insertItem(tr("Un&hide"), this, SLOT(trayShow()));
-	else
+	}
+	else {
 		d->trayMenu->insertItem(tr("&Hide"), this, SLOT(trayHide()));
+	}
 	d->optionsButton->addTo(d->trayMenu);
 	d->trayMenu->insertItem(tr("Status"), d->statusMenu);
 	
@@ -815,8 +876,9 @@ void MainWin::buildTrayMenu()
 
 void MainWin::setTrayToolTip(int status)
 {
-	if (!d->tray)
+	if (!d->tray) {
 		return;
+	}
 	d->tray->setToolTip(QString("Psi - " + status2txt(status)));
 }
 
@@ -825,17 +887,18 @@ void MainWin::decorateButton(int status)
 	// update the 'change status' buttons
 	QObjectList l = d->statusGroup->queryList( "IconAction" );
 	for (QObjectList::Iterator it = l.begin() ; it != l.end(); ++it) {
-		IconAction *action = (IconAction *)(*it);
+		IconAction* action = (IconAction *)(*it);
 		action->setChecked ( d->statusActions[action] == status );
 	}
 
-	if(d->lastStatus == status)
+	if(d->lastStatus == status) {
 		return;
+	}
 	d->lastStatus = status;
 
 	if(status == -1) {
 		d->statusButton->setText(tr("Connecting"));
-		if (option.alertStyle != 0) {
+		if (PsiOptions::instance()->getOption("options.ui.notifications.alert-style").toString() != "no") {
 			d->statusButton->setAlert(IconsetFactory::iconPtr("psi/connect"));
 			d->statusGroup->setPsiIcon(IconsetFactory::iconPtr("psi/connect"));
 		}
@@ -869,11 +932,12 @@ void MainWin::try2tryCloseProgram()
 
 void MainWin::tryCloseProgram()
 {
-	if(askQuit())
+	if(askQuit()) {
 		closeProgram();
+	}
 }
 
-void MainWin::closeEvent(QCloseEvent *e)
+void MainWin::closeEvent(QCloseEvent* e)
 {
 #ifdef Q_WS_MAC
 	trayHide();
@@ -885,17 +949,17 @@ void MainWin::closeEvent(QCloseEvent *e)
 		return;
 	}
 
-	if(!askQuit())
+	if(!askQuit()) {
 		return;
+	}
 
-	emit geomChanged(saveableGeometry());
 	closeProgram();
 
 	e->accept();
 #endif
 }
 
-void MainWin::keyPressEvent(QKeyEvent *e)
+void MainWin::keyPressEvent(QKeyEvent* e)
 {
 #ifdef Q_WS_MAC
 	bool allowed = true;
@@ -904,11 +968,17 @@ void MainWin::keyPressEvent(QKeyEvent *e)
 #endif
 
 	bool closekey = false;
-	if(e->key() == Qt::Key_Escape)
-		closekey = true;
+	if(e->key() == Qt::Key_Escape) {
+		if (d->searchWidget->isVisible()) {
+			searchClearClicked();
+		} else {
+			closekey = true;
+		}
+	}
 #ifdef Q_WS_MAC
-	else if(e->key() == Qt::Key_W && e->modifiers() & Qt::ControlModifier)
+	else if(e->key() == Qt::Key_W && e->modifiers() & Qt::ControlModifier) {
 		closekey = true;
+	}
 #endif
 
 	if(allowed && closekey) {
@@ -922,7 +992,7 @@ void MainWin::keyPressEvent(QKeyEvent *e)
 
 #ifdef Q_WS_WIN
 #include <windows.h>
-bool MainWin::winEvent(MSG *msg, long *result)
+bool MainWin::winEvent(MSG* msg, long* result)
 {
 	if (d->asTool && msg->message == WM_SYSCOMMAND && msg->wParam == SC_MINIMIZE) {
 		hide();	// minimized toolwindows look bad on Windows, so let's just hide it instead
@@ -938,16 +1008,20 @@ void MainWin::updateCaption()
 {
 	QString str = "";
 
-	if(d->nextAmount > 0)
+	if(d->nextAmount > 0) {
 		str += "* ";
+	}
 
-	if(d->nickname.isEmpty())
+	if(d->nickname.isEmpty()) {
 		str += ApplicationInfo::name();
-	else
+	}
+	else {
 		str += d->nickname;
+	}
 
-	if(str == caption())
+	if(str == caption()) {
 		return;
+	}
 
 	setWindowTitle(str);
 }
@@ -959,12 +1033,14 @@ void MainWin::optionsUpdate()
 	decorateButton(status);
 
 #ifndef Q_WS_MAC
-	if (option.hideMenubar) 
+	if (!PsiOptions::instance()->getOption("options.ui.contactlist.show-menubar").toBool()) {
 		mainMenuBar()->hide();
-	else 
+	}
+	else {
 		mainMenuBar()->show();
+	}
 #endif
-	
+
 	setWindowOpacity(double(qMax(MINIMUM_OPACITY,PsiOptions::instance()->getOption("options.ui.contactlist.opacity").toInt()))/100);
 
 	buildStatusMenu();
@@ -974,16 +1050,19 @@ void MainWin::optionsUpdate()
 
 void MainWin::toggleVisible()
 {
-	if(!isHidden())
+	if(!isHidden()) {
 		trayHide();
-	else
+	}
+	else {
 		trayShow();
+	}
 }
 
-void MainWin::setTrayToolTip(const Status &status, bool)
+void MainWin::setTrayToolTip(const Status& status, bool)
 {
-	if (!d->tray)
+	if (!d->tray) {
 		return;
+	}
 	QString s = "Psi";
 
  	QString show = status.show();
@@ -993,32 +1072,37 @@ void MainWin::setTrayToolTip(const Status &status, bool)
 	}
 
 	QString text = status.status();
-	if(!text.isEmpty())
+	if(!text.isEmpty()) {
 		s += ": "+text;
+	}
 
 	d->tray->setToolTip(s);
 }
 
 void MainWin::trayClicked(const QPoint &, int button)
 {
-	if(option.dockDCstyle)
+	if(PsiOptions::instance()->getOption("options.ui.systemtray.use-double-click").toBool()) {
 		return;
+	}
 
 	if(button == Qt::MidButton) {
 		doRecvNextEvent();
 		return;
 	}
 
-	if(!isHidden())
+	if(!isHidden()) {
 		trayHide();
-	else
+	}
+	else {
 		trayShow();
+	}
 }
 
 void MainWin::trayDoubleClicked()
 {
-	if(!option.dockDCstyle)
+	if(!PsiOptions::instance()->getOption("options.ui.systemtray.use-double-click").toBool()) {
 		return;
+	}
 
 	if(d->nextAmount > 0) {
 		doRecvNextEvent();
@@ -1026,10 +1110,12 @@ void MainWin::trayDoubleClicked()
 	}
 
 
-	if(!isHidden())
+	if(!isHidden()) {
 		trayHide();
-	else
+	}
+	else {
 		trayShow();
+	}
 }
 
 void MainWin::trayShow()
@@ -1039,24 +1125,25 @@ void MainWin::trayShow()
 
 void MainWin::trayHide()
 {
-	emit geomChanged(saveableGeometry());
 	hide();
 }
 
-void MainWin::updateReadNext(PsiIcon *anim, int amount)
+void MainWin::updateReadNext(PsiIcon* anim, int amount)
 {
 	d->nextAnim = anim;
-	if(anim == 0)
+	if(anim == 0) {
 		d->nextAmount = 0;
-	else
+	}
+	else {
 		d->nextAmount = amount;
+	}
 
 	if(d->nextAmount <= 0) {
 		d->eventNotifier->hide();
-		d->eventNotifier->setText("");
+		d->eventNotifier->setMessage("");
 	}
 	else {
-		d->eventNotifier->setText(QString("<b>") + numEventsString(d->nextAmount) + "</b>");
+		d->eventNotifier->setMessage(QString("<b>") + numEventsString(d->nextAmount) + "</b>");
 		d->eventNotifier->show();
 		// make sure it shows
 		//qApp->processEvents();
@@ -1069,27 +1156,34 @@ void MainWin::updateReadNext(PsiIcon *anim, int amount)
 QString MainWin::numEventsString(int x) const
 {
 	QString s;
-	if(x <= 0)
+	if(x <= 0) {
 		s = "";
-	else if(x == 1)
+	}
+	else if(x == 1) {
 		s = tr("1 event received");
-	else
+	}
+	else {
 		s = tr("%1 events received").arg(x);
+	}
 
 	return s;
 }
 
 void MainWin::updateTray()
 {
-	if(!d->tray)
+	if(!d->tray) {
 		return;
+	}
 
-	if ( d->nextAmount > 0 )
+	if ( d->nextAmount > 0 ) {
 		d->tray->setAlert(d->nextAnim);
-	else if ( d->lastStatus == -1 )
+	}
+	else if ( d->lastStatus == -1 ) {
 		d->tray->setAlert(IconsetFactory::iconPtr("psi/connect"));
-	else
+	}
+	else {
 		d->tray->setIcon(PsiIconset::instance()->statusPtr(d->lastStatus));
+	}
 	
 	buildTrayMenu();
 	d->tray->setContextMenu(d->trayMenu);
@@ -1102,8 +1196,9 @@ void MainWin::doRecvNextEvent()
 
 void MainWin::statusClicked(int x)
 {
-	if(x == Qt::MidButton)
+	if(x == Qt::MidButton) {
 		recvNextEvent();
+	}
 }
 
 void MainWin::numAccountsChanged()
@@ -1128,10 +1223,61 @@ void MainWin::accountFeaturesChanged()
 
 void MainWin::dockActivated()
 {
-	if(isHidden())
+	if(isHidden()) {
 		show();
+	}
 }
 
+/**
+ * Called when the cancel is clicked or the search becomes empty.
+ * Cancels the search.
+ */ 
+void MainWin::searchClearClicked()
+{
+	d->searchWidget->setVisible(false);
+	d->searchText->clear();
+	cvlist->clearFilter();
+	if (d->filterActive)
+	{
+		d->getAction("show_offline")->setChecked(d->prefilterShowOffline);
+		d->getAction("show_away")->setChecked(d->prefilterShowAway);  
+	}
+	d->filterActive=false;  
+}
+
+/**
+ * Called when the contactview has a keypress.
+ * Starts the search/filter process
+ */ 
+void MainWin::searchTextStarted(QString const& text)
+{
+	d->searchWidget->setVisible(true);
+	d->searchText->setText(d->searchText->text() + text);
+	searchTextEntered(d->searchText->text());
+	d->searchText->setFocus();
+}
+
+/**
+ * Called when the search input is changed.
+ * Updates the search.
+ */ 
+void MainWin::searchTextEntered(QString const& text)
+{
+	if (!d->filterActive)
+	{
+		d->filterActive = true;
+		d->prefilterShowOffline = d->getAction("show_offline")->isChecked();
+		d->prefilterShowAway = d->getAction("show_away")->isChecked();
+		d->getAction("show_offline")->setChecked(true);
+		d->getAction("show_away")->setChecked(true);
+	}
+	if (text.isEmpty()) {
+		searchClearClicked();
+	} else {
+		
+		cvlist->setFilter(text);
+	}
+}
 
 #ifdef Q_WS_MAC
 void MainWin::setWindowIcon(const QPixmap&)
@@ -1140,7 +1286,7 @@ void MainWin::setWindowIcon(const QPixmap&)
 #else
 void MainWin::setWindowIcon(const QPixmap& p)
 {
-	Q3MainWindow::setWindowIcon(p);
+	QMainWindow::setWindowIcon(p);
 }
 #endif
 
@@ -1204,8 +1350,8 @@ void MainWin::showNoFocus()
 
 	if ( children() ) {
 		QObjectListIt it(*children());
-		register QObject *object;
-		QWidget *widget;
+		register QObject* object;
+		QWidget* widget;
 		while ( it ) {				// show all widget children
 			object = it.current();		//   (except popups and other toplevels)
 			++it;
