@@ -18,12 +18,11 @@
  *
  */
 
-#include "common.h"
 #include "mainwin_p.h"
 
 #include <qapplication.h>
 #include <qstyle.h>
-#include <q3toolbar.h>
+#include <QToolBar>
 #include <qtimer.h>
 #include <qsignalmapper.h>
 #include <qobject.h>
@@ -35,6 +34,8 @@
 #include <QLabel>
 #include <QMenu>
 #include <QMouseEvent>
+#include <QtAlgorithms>
+
 #include "psiaccount.h"
 #include "stretchwidget.h"
 #include "iconwidget.h"
@@ -55,7 +56,10 @@ public:
 
 	void setIcon(PsiIcon *, bool showText);
 	void setLabel(QString);
+
+	// reimplemented
 	QSize sizeHint() const;
+	QSize minimumSizeHint() const;
 
 private slots:
 	void pixmapUpdated();
@@ -89,6 +93,11 @@ QSize PopupActionButton::sizeHint() const
 	return QPushButton::sizeHint();
 }
 
+QSize PopupActionButton::minimumSizeHint() const
+{
+	return QSize(16, 16);
+}
+
 void PopupActionButton::setIcon(PsiIcon *i, bool st)
 {
 	if ( icon ) {
@@ -116,10 +125,21 @@ void PopupActionButton::setLabel(QString lbl)
 
 void PopupActionButton::update()
 {
-	if ( showText )
+	if(qVersionInt() >= 0x040300)
+	{
+		if((showText && !label.isEmpty()) && styleSheet().isEmpty()) {
+			setStyleSheet("text-align: left");
+		}
+		else if((!showText || label.isEmpty()) && styleSheet() == "text-align: left") {
+			setStyleSheet(QString());
+		}
+	}
+
+	if (showText) {
 		QPushButton::setText(label);
-	else
+	} else {
 		QPushButton::setText("");
+	}
 }
 
 void PopupActionButton::pixmapUpdated()
@@ -289,22 +309,25 @@ void PopupAction::setText (const QString &text)
 	}
 }
 
-bool PopupAction::addTo (QWidget *w)
+bool PopupAction::addTo(QWidget *w)
 {
-	if ( w->inherits("QToolBar") || w->inherits("Q3ToolBar") ) {
-		QByteArray bname((const char*) (QString(name()) + QString("_action_button")));
-		PopupActionButton *btn = new PopupActionButton ( (Q3ToolBar*)w, bname );
-		d->buttons.append ( btn );
-		btn->setMenu ( menu() );
-		btn->setLabel ( text() );
-		btn->setIcon ( d->icon, d->showText );
-		btn->setSizePolicy ( d->size );
-		btn->setEnabled ( isEnabled() );
+	QToolBar* toolbar = dynamic_cast<QToolBar*>(w);
+	if (toolbar) {
+		QByteArray bname((const char*)(QString(name()) + QString("_action_button")));
+		PopupActionButton *btn = new PopupActionButton(w, bname);
+		d->buttons.append(btn);
+		btn->setMenu(menu());
+		btn->setLabel(text());
+		btn->setIcon(d->icon, d->showText);
+		btn->setSizePolicy(d->size);
+		btn->setEnabled(isEnabled());
+		toolbar->addWidget(btn);
 
-		connect( btn, SIGNAL( destroyed() ), SLOT( objectDestroyed() ) );
+		connect(btn, SIGNAL(destroyed()), SLOT(objectDestroyed()));
 	}
-	else
+	else {
 		return IconAction::addTo(w);
+	}
 
 	return true;
 }
@@ -375,127 +398,75 @@ void MLabel::mouseDoubleClickEvent(QMouseEvent *e)
 // MAction
 //----------------------------------------------------------------------------
 
-class MAction::Private : public QObject
+MAction::MAction(PsiIcon i, const QString& s, int id, PsiCon* psi, QObject* parent)
+	: IconActionGroup(parent)
 {
-public:
-	int id;
-	PsiCon *psi;
-	QSignalMapper *sm;
-
-	Private (int _id, PsiCon *_psi, QObject *parent)
-	: QObject (parent)
-	{
-		id = _id;
-		psi = _psi;
-		sm = new QSignalMapper(this, "MAction::Private::SignalMapper");
-	}
-
-	QMenu *subMenu(QWidget *p)
-	{
-		QMenu *pm = new QMenu (p);
-		uint i = 0;
-		foreach(PsiAccount* acc, psi->contactList()->enabledAccounts()) {
-			pm->insertItem( acc->name(), parent(), SLOT(itemActivated(int)), 0, id*1000 + i );
-			pm->setItemParameter ( id*1000 + i, i );
-			i++;
-		}
-		return pm;
-	}
-
-	void updateToolButton(QToolButton *btn)
-	{
-		if (psi->contactList()->enabledAccounts().count() >= 2) {
-			btn->setMenu(subMenu(btn));
-			disconnect(btn, SIGNAL(clicked()), sm, SLOT(map()));
-		}
-		else {
-			btn->setMenu(0);
-			// connect exactly once:
-			disconnect(btn, SIGNAL(clicked()), sm, SLOT(map()));
-			connect(btn, SIGNAL(clicked()), sm, SLOT(map()));
-		}
-	}
-};
-
-MAction::MAction(PsiIcon i, const QString &s, int id, PsiCon *psi, QObject *parent)
-: IconAction(s, s, 0, parent)
-{
-	init (i, id, psi);
+	init(s, i, id, psi);
 }
 
-MAction::MAction(const QString &s, int id, PsiCon *psi, QObject *parent)
-: IconAction(s, s, 0, parent)
+MAction::MAction(const QString& s, int id, PsiCon* psi, QObject* parent)
+	: IconActionGroup(parent)
 {
-	init (PsiIcon(), id, psi);
+	init(s, PsiIcon(), id, psi);
 }
 
-void MAction::init(PsiIcon i, int id, PsiCon *psi)
+void MAction::init(const QString& name, PsiIcon i, int id, PsiCon* psi)
 {
-	d = new Private(id, psi, this);
-	setPsiIcon (&i);
-	connect(psi, SIGNAL(accountCountChanged()), SLOT(numAccountsChanged()));
-	setEnabled ( d->psi->contactList()->haveEnabledAccounts() );
-	connect (d->sm, SIGNAL(mapped(int)), SLOT(itemActivated(int)));
+	id_ = id;
+	controller_ = psi;
+
+	setText(name);
+	setStatusTip(name);
+	setPsiIcon(&i);
+
+	connect(controller_, SIGNAL(accountCountChanged()), SLOT(numAccountsChanged()));
+	connect(this, SIGNAL(activated()), SLOT(slotActivated()));
+	numAccountsChanged();
 }
 
-bool MAction::addTo(QWidget *w)
+bool MAction::addTo(QWidget* widget)
 {
-	if ( w->inherits("QPopupMenu") || w->inherits("QMenu") )
-	{
-		QMenu *menu = (QMenu*)w;
-        QIcon iconset;
-#ifndef Q_WS_MAC
-		iconset = iconSet();
-#endif
-		if ( d->psi->contactList()->enabledAccounts().count() < 2 ) {
-			menu->insertItem ( iconset, menuText(), this, SLOT(itemActivated(int)), 0, d->id*1000 + 0 );
-			menu->setItemEnabled (d->id*1000 + 0, isEnabled());
-			menu->setItemParameter ( d->id*1000 + 0, 0 );
-		}
-		else
-			menu->insertItem(iconset, menuText(), d->subMenu(w));
-	}
-	else
-		return IconAction::addTo(w);
-
+	widget->addAction(this);
 	return true;
 }
 
-void MAction::addingToolButton(IconToolButton *btn)
+QList<PsiAccount*> MAction::accounts() const
 {
-	d->sm->setMapping(btn, 0);
-
-	d->updateToolButton(btn);
+	return controller_->contactList()->enabledAccounts();
 }
 
-void MAction::itemActivated(int n)
+void MAction::slotActivated()
 {
-	QList<PsiAccount*> list = d->psi->contactList()->enabledAccounts();
+	if (accounts().count() > 0) {
+		emit activated(accounts().first(), id_);
+	}
+}
 
-	if (n >= list.count()) // just in case
-		return;
-
-	emit activated((PsiAccount *)list.at(n), d->id);
+void MAction::actionActivated()
+{
+	QAction* action = static_cast<QAction*>(sender());
+	int num = action->property("id").toInt();
+	if (num >= 0 && num < accounts().count()) {
+		emit activated(accounts().at(num), id_);
+	}
 }
 
 void MAction::numAccountsChanged()
 {
-	setEnabled( d->psi->contactList()->haveEnabledAccounts() );
+	setEnabled(accounts().count() > 0);
 
-	QList<IconToolButton*> btns = buttonList();
-	for ( QList<IconToolButton*>::Iterator it = btns.begin(); it != btns.end(); ++it ) {
-		QToolButton *btn = (*it);
+	qDeleteAll(findChildren<QAction*>());
 
-		if (btn->menu())
-			delete btn->menu();
-
-		d->updateToolButton(btn);
+	foreach(PsiAccount* account, accounts()) {
+		QAction* act = new QAction(account->name(), this);
+		act->setProperty("id", accounts().indexOf(account));
+		connect(act, SIGNAL(activated()), SLOT(actionActivated()));
 	}
 }
 
 IconAction *MAction::copy() const
 {
-	MAction *act = new MAction(text(), d->id, d->psi, 0);
+	MAction *act = new MAction(text(), id_, controller_, 0);
 
 	*act = *this;
 
@@ -509,6 +480,11 @@ MAction &MAction::operator=( const MAction &from )
 	return *this;
 }
 
+void MAction::doSetMenu(QMenu* menu)
+{
+	IconActionGroup::doSetMenu(findChildren<QAction*>().count() > 1 ? menu : 0);
+}
+
 //----------------------------------------------------------------------------
 // SpacerAction
 //----------------------------------------------------------------------------
@@ -527,8 +503,10 @@ SpacerAction::~SpacerAction()
 
 bool SpacerAction::addTo(QWidget *w)
 {
-	if ( w->inherits("QToolBar") || w->inherits("Q3ToolBar") ) {
-		new StretchWidget(w);
+	QToolBar* toolbar = dynamic_cast<QToolBar*>(w);
+	if (toolbar) {
+		StretchWidget* stretch = new StretchWidget(w);
+		toolbar->addWidget(stretch);
 		return true;
 	}
 
@@ -579,13 +557,14 @@ public:
 
 	Q3PtrList<MLabel> labels;
 	bool hide;
+	QString message;
 };
 
 EventNotifierAction::EventNotifierAction(QObject *parent, const char *name)
 : IconAction(parent, name)
 {
 	d = new Private;
-	setMenuText(tr("<Event notifier>"));
+	setText(tr("<Event notifier>"));
 	d->hide = true;
 }
 
@@ -596,15 +575,25 @@ EventNotifierAction::~EventNotifierAction()
 
 bool EventNotifierAction::addTo(QWidget *w)
 {
-	if ( w->inherits("QToolBar") || w->inherits("Q3ToolBar") ) {
+	if (w) {
 		MLabel *label = new MLabel(w, "EventNotifierAction::MLabel");
-		label->setText(text());
+		label->setText(d->message);
 		d->labels.append(label);
 		connect(label, SIGNAL(destroyed()), SLOT(objectDestroyed()));
 		connect(label, SIGNAL(doubleClicked()), SIGNAL(activated()));
 		connect(label, SIGNAL(clicked(int)), SIGNAL(clicked(int)));
 
-		if ( d->hide )
+		QToolBar* toolbar = dynamic_cast<QToolBar*>(w);
+		if (!toolbar) {
+			QLayout* layout = w->layout();
+			if (layout)
+				layout->addWidget(label);
+		}
+		else {
+			toolbar->addWidget(label);
+		}
+
+		if (d->hide)
 			hide();
 
 		return true;
@@ -613,14 +602,14 @@ bool EventNotifierAction::addTo(QWidget *w)
 	return false;
 }
 
-void EventNotifierAction::setText(const QString &t)
+void EventNotifierAction::setMessage(const QString &m)
 {
-	IconAction::setText("<nobr>" + t + "</nobr>");
+	d->message = m;
 
 	Q3PtrListIterator<MLabel> it ( d->labels );
 	for ( ; it.current(); ++it) {
 		MLabel *label = it.current();
-		label->setText(text());
+		label->setText(d->message);
 	}
 }
 
@@ -634,22 +623,24 @@ void EventNotifierAction::hide()
 {
 	d->hide = true;
 
-	Q3PtrListIterator<MLabel> it ( d->labels );
-	for ( ; it.current(); ++it) {
+	Q3PtrListIterator<MLabel> it(d->labels);
+	for (; it.current(); ++it) {
 		MLabel *label = it.current();
 		label->hide();
-		Q3ToolBar *toolBar = (Q3ToolBar *)label->parent();
+		QToolBar *toolBar = dynamic_cast<QToolBar*>(label->parent());
+		if (toolBar) {
+			int found = 0;
+			foreach(QWidget* widget, toolBar->findChildren<QWidget*>()) {
+				if (!widget->objectName().startsWith("qt_") &&
+				    !QString(widget->metaObject()->className()).startsWith("QToolBar"))
+				{
+					found++;
+				}
+			}
 
-		QObjectList l = toolBar->queryList( "QWidget" );
-		int found = 0;
-
-		for ( QObjectList::ConstIterator it = l.begin(); it != l.end(); ++it) {
-			if ( QString((*it)->name()).left(3) != "qt_" ) // misc internal Qt objects
-				found++;
+			if (found == 1)   // only MLabel is on ToolBar
+				toolBar->hide();
 		}
-
-		if ( found == 1 ) // only MLabel is on ToolBar
-			toolBar->hide();
 	}
 }
 
@@ -660,12 +651,21 @@ void EventNotifierAction::show()
 	Q3PtrListIterator<MLabel> it ( d->labels );
 	for ( ; it.current(); ++it) {
 		MLabel *label = it.current();
-		Q3ToolBar *toolBar = (Q3ToolBar *)label->parent();
 		label->show();
-		toolBar->show();
+		QToolBar *toolBar = dynamic_cast<QToolBar*>(label->parent());
+		if (toolBar)
+			toolBar->show();
 	}
 }
 
+void EventNotifierAction::updateVisibility()
+{
+	if (d->hide)
+		hide();
+	else
+		show();
+}
+
 IconAction *EventNotifierAction::copy() const
 {
 	EventNotifierAction *act = new EventNotifierAction( 0 );
