@@ -1,6 +1,7 @@
 #include "opt_status.h"
 #include "common.h"
 #include "iconwidget.h"
+#include "psioptions.h"
 
 #include <qbuttongroup.h>
 #include <QMessageBox>
@@ -25,18 +26,14 @@ public:
 // OptionsTabStatus
 //----------------------------------------------------------------------------
 
-static XMPP::Status::Type combomap[7] = { STATUS_CHAT, STATUS_ONLINE, STATUS_AWAY, STATUS_XA, STATUS_DND, STATUS_INVISIBLE, STATUS_OFFLINE };
-
 OptionsTabStatus::OptionsTabStatus(QObject *parent)
 : OptionsTab(parent, "status", "", tr("Status"), tr("Status preferences"), "psi/status")
 {
 	w = 0;
-	o = new Options;
 }
 
 OptionsTabStatus::~OptionsTabStatus()
 {
-	delete o;
 }
 
 QWidget *OptionsTabStatus::widget()
@@ -58,6 +55,11 @@ QWidget *OptionsTabStatus::widget()
 	s = tr("Makes Psi automatically set your status to \"offline\" if your"
 		" computer is idle for the specified amount of time."
 		"  This will disconnect you from the Jabber server.");
+	if (!PsiOptions::instance()->getOption("options.ui.menu.status.xa").toBool()) {
+		d->ck_asXa->hide();
+		d->sb_asXa->hide();
+		d->lb_asXa->hide();
+	}
 	QWhatsThis::add(d->ck_asOffline, s);
 	QWhatsThis::add(d->sb_asOffline, s);
 
@@ -65,20 +67,14 @@ QWidget *OptionsTabStatus::widget()
 		tr("Specifies an extended message to use if you allow Psi"
 		" to set your status automatically.  See options above."));
 
-	d->pb_spNew->setEnabled(TRUE);
-	d->pb_spDelete->setEnabled(FALSE);
-	d->te_sp->setEnabled(FALSE);
+	setStatusPresetWidgetsEnabled(false);
 	connect(d->pb_spNew, SIGNAL(clicked()), SLOT(newStatusPreset()));
 	connect(d->pb_spDelete, SIGNAL(clicked()), SLOT(removeStatusPreset()));
-	connect(d->cb_preset, SIGNAL(highlighted(int)), SLOT(selectStatusPreset(int)));
+	connect(d->cb_preset, SIGNAL(currentIndexChanged(int)), SLOT(selectStatusPreset(int)));
 	connect(d->te_sp, SIGNAL(textChanged()), SLOT(changeStatusPreset()));
 	connect(d->le_sp_priority, SIGNAL(textChanged(const QString&)), SLOT(changeStatusPreset()));
 	connect(d->cb_sp_status, SIGNAL(activated(int)), SLOT(changeStatusPreset()));
 	
-	int n;
-	for(n = 0; n < 7; ++n)
-		d->cb_sp_status->insertItem(status2txt(combomap[n]));
-
 	QWhatsThis::add(d->pb_spNew,
 		tr("Press this button to create a new status message preset."));
 	QWhatsThis::add(d->pb_spDelete,
@@ -104,28 +100,46 @@ QWidget *OptionsTabStatus::widget()
 	return w;
 }
 
-void OptionsTabStatus::applyOptions(Options *opt)
+void OptionsTabStatus::applyOptions()
 {
 	if ( !w )
 		return;
 
 	OptStatusUI *d = (OptStatusUI *)w;
 
-	opt->asAway = d->sb_asAway->value();
-	opt->asXa = d->sb_asXa->value();
-	opt->asOffline = d->sb_asOffline->value();
-	opt->use_asAway = d->ck_asAway->isChecked();
-	opt->use_asXa = d->ck_asXa->isChecked();
-	opt->use_asOffline = d->ck_asOffline->isChecked();
-	opt->asMessage = d->te_asMessage->text();
+	PsiOptions::instance()->setOption("options.status.auto-away.away-after", d->sb_asAway->value());
+	PsiOptions::instance()->setOption("options.status.auto-away.not-availible-after", d->sb_asXa->value());
+	PsiOptions::instance()->setOption("options.status.auto-away.offline-after", d->sb_asOffline->value());
+	PsiOptions::instance()->setOption("options.status.auto-away.use-away", d->ck_asAway->isChecked());
+	PsiOptions::instance()->setOption("options.status.auto-away.use-not-availible", d->ck_asXa->isChecked());
+	PsiOptions::instance()->setOption("options.status.auto-away.use-offline", d->ck_asOffline->isChecked());
+	PsiOptions::instance()->setOption("options.status.auto-away.message", d->te_asMessage->text());
 
-	opt->sp = o->sp;
-
-	opt->askOnline = d->ck_askOnline->isChecked();
-	opt->askOffline = d->ck_askOffline->isChecked();
+	
+	foreach (QString name, deletedPresets) {
+		QString base = PsiOptions::instance()->mapLookup("options.status.presets", name);
+		PsiOptions::instance()->removeOption(base , true);
+	}
+	deletedPresets.clear();
+	foreach (QString name, dirtyPresets.toList() + newPresets.keys()) {
+		StatusPreset sp;
+		if (newPresets.contains(name)) {
+			sp = newPresets[name];
+		} else {
+			sp = presets[name];
+		}
+		PsiOptions *o = PsiOptions::instance();
+		sp.toOptions(o);
+	}
+	dirtyPresets.clear();
+	presets.unite(newPresets);
+	newPresets.clear();
+	
+	PsiOptions::instance()->setOption("options.status.ask-for-message-on-online", d->ck_askOnline->isChecked());
+	PsiOptions::instance()->setOption("options.status.ask-for-message-on-offline", d->ck_askOffline->isChecked());
 }
 
-void OptionsTabStatus::restoreOptions(const Options *opt)
+void OptionsTabStatus::restoreOptions()
 {
 	if ( !w )
 		return;
@@ -133,37 +147,52 @@ void OptionsTabStatus::restoreOptions(const Options *opt)
 	OptStatusUI *d = (OptStatusUI *)w;
 
 	d->sb_asAway->setMinValue(0);
-	d->sb_asAway->setValue( opt->asAway );
+	d->sb_asAway->setValue( PsiOptions::instance()->getOption("options.status.auto-away.away-after").toInt() );
 	d->sb_asXa->setMinValue(0);
-	d->sb_asXa->setValue( opt->asXa );
+	d->sb_asXa->setValue( PsiOptions::instance()->getOption("options.status.auto-away.not-availible-after").toInt() );
 	d->sb_asOffline->setMinValue(0);
-	d->sb_asOffline->setValue( opt->asOffline );
-	/*if (opt->asAway <= 0 )
-		opt->use_asAway = FALSE;
-	if (opt->asXa <= 0 )
-		opt->use_asXa = FALSE;
+	d->sb_asOffline->setValue( PsiOptions::instance()->getOption("options.status.auto-away.offline-after").toInt() );
+	/*if (PsiOptions::instance()->getOption("options.status.auto-away.away-after").toInt() <= 0 )
+		PsiOptions::instance()->getOption("options.status.auto-away.use-away").toBool() = FALSE;
+	if (PsiOptions::instance()->getOption("options.status.auto-away.not-availible-after").toInt() <= 0 )
+		PsiOptions::instance()->getOption("options.status.auto-away.use-not-availible").toBool() = FALSE;
 	if(d->opt.asOffline <= 0)
-		opt->use_asOffline = FALSE;*/
-	d->ck_asAway->setChecked( opt->use_asAway );
-	d->ck_asXa->setChecked( opt->use_asXa );
-	d->ck_asOffline->setChecked( opt->use_asOffline );
-	d->te_asMessage->setText( opt->asMessage );
+		PsiOptions::instance()->getOption("options.status.auto-away.use-offline").toBool() = FALSE;*/
+	d->ck_asAway->setChecked( PsiOptions::instance()->getOption("options.status.auto-away.use-away").toBool() );
+	d->ck_asXa->setChecked( PsiOptions::instance()->getOption("options.status.auto-away.use-not-availible").toBool() );
+	d->ck_asOffline->setChecked( PsiOptions::instance()->getOption("options.status.auto-away.use-offline").toBool() );
+	d->te_asMessage->setText( PsiOptions::instance()->getOption("options.status.auto-away.message").toString() );
 
-	o->sp = opt->sp;
-	d->cb_preset->clear();
-	QStringList presets;
-	foreach(StatusPreset p, option.sp) {
-		presets += p.name();
+	
+	QStringList presetNames;
+	
+	foreach(QVariant name, PsiOptions::instance()->mapKeyList("options.status.presets")) {
+		QString base =  PsiOptions::instance()->mapLookup("options.status.presets", name.toString());
+		StatusPreset sp;
+		sp.setName(name.toString());
+		sp.setMessage(PsiOptions::instance()->getOption(base+".message").toString());
+		if (PsiOptions::instance()->getOption(base+".force-priority").toBool()) {
+			sp.setPriority(PsiOptions::instance()->getOption(base+".priority").toInt());
+		}
+
+		XMPP::Status status;
+		status.setType(PsiOptions::instance()->getOption(base+".status").toString());
+		sp.setStatus(status.type());
+		
+		presets[name.toString()] = sp;
+		presetNames += name.toString();
 	}
-	d->cb_preset->insertStringList(presets);
+	
+	
+	d->cb_preset->insertStringList(presetNames);
 
 	if(d->cb_preset->count() >= 1) {
 		d->cb_preset->setCurrentIndex(0);
 		selectStatusPreset(0);
 	}
 
-	d->ck_askOnline->setChecked( opt->askOnline );
-	d->ck_askOffline->setChecked( opt->askOffline );
+	d->ck_askOnline->setChecked( PsiOptions::instance()->getOption("options.status.ask-for-message-on-online").toBool() );
+	d->ck_askOffline->setChecked( PsiOptions::instance()->getOption("options.status.ask-for-message-on-offline").toBool() );
 }
 
 void OptionsTabStatus::setData(PsiCon *, QWidget *parentDialog)
@@ -179,12 +208,9 @@ void OptionsTabStatus::selectStatusPreset(int x)
 	disconnect(d->te_sp, SIGNAL(textChanged()), 0, 0);
 	disconnect(d->le_sp_priority, SIGNAL(textChanged(const QString&)), 0, 0);
 	if ( x == -1 ) {
-		d->pb_spDelete->setEnabled(false);
+		setStatusPresetWidgetsEnabled(false);
 		d->te_sp->setText("");
-		d->te_sp->setEnabled(false);
 		d->le_sp_priority->clear();
-		d->le_sp_priority->setEnabled(false);
-		d->cb_sp_status->setEnabled(false);
 
 		//noDirty = FALSE;
 		connect(d->te_sp, SIGNAL(textChanged()), SLOT(changeStatusPreset()));
@@ -192,28 +218,27 @@ void OptionsTabStatus::selectStatusPreset(int x)
 		return;
 	}
 
-	d->pb_spDelete->setEnabled(true);
-
-	StatusPreset preset = o->sp[d->cb_preset->text(x)];
+	StatusPreset preset;
+	QString name = d->cb_preset->text(x);
+	
+	if (newPresets.contains(name)) {
+		preset = newPresets[name];
+	} else {
+		preset = presets[name];
+	}
+	
 	d->te_sp->setText(preset.message());
-	d->te_sp->setEnabled(true);
 	if (preset.priority().hasValue())
 		d->le_sp_priority->setText(QString::number(preset.priority().value()));
 	else
 		d->le_sp_priority->clear();
-	d->le_sp_priority->setEnabled(true);
-	int n;
-	for(n = 0; n < 7; ++n) {
-		if(preset.status() == combomap[n]) {
-			d->cb_sp_status->setCurrentItem(n);
-			break;
-		}
-	}
-	d->cb_sp_status->setEnabled(true);
-
+	d->cb_sp_status->setStatus(preset.status());
+	
 	//noDirty = FALSE;
 	connect(d->te_sp, SIGNAL(textChanged()), SLOT(changeStatusPreset()));
 	connect(d->le_sp_priority, SIGNAL(textChanged(const QString&)), SLOT(changeStatusPreset()));
+	
+	setStatusPresetWidgetsEnabled(true);
 }
 
 void OptionsTabStatus::newStatusPreset()
@@ -228,18 +253,20 @@ void OptionsTabStatus::newStatusPreset()
 			CAP(tr("New Status Preset")),
 			tr("Please enter a name for the new status preset:"),
 			QLineEdit::Normal, text, &ok, parentWidget);
-		if(!ok)
+		if(!ok) {
 			return;
+		}
 
-		if(text.isEmpty())
+		if(text.isEmpty()) {
 			QMessageBox::information(parentWidget, tr("Error"), tr("Can't create a blank preset!"));
-		else if(o->sp.contains(text))
+		} else if(presets.contains(text) || newPresets.contains(text)) {
 			QMessageBox::information(parentWidget, tr("Error"), tr("You already have a preset with that name!"));
-		else
+		} else {
 			break;
+		}
 	}
 
-	o->sp[text].setName(text);
+	newPresets[text].setName(text);
 	d->cb_preset->insertItem(text);
 	d->cb_preset->setCurrentItem(d->cb_preset->count()-1);
 	selectStatusPreset(d->cb_preset->count()-1);
@@ -257,7 +284,14 @@ void OptionsTabStatus::removeStatusPreset()
 
 	emit dataChanged();
 
-	o->sp.remove(d->cb_preset->text(id));
+	QString name = d->cb_preset->text(id);
+	
+	if (newPresets.contains(name)) {
+		newPresets.remove(name);
+	} else {
+		deletedPresets += d->cb_preset->text(id);
+		presets.remove(d->cb_preset->text(id));
+	}
 	d->cb_preset->removeItem(id);
 
 	// select a new entry if possible
@@ -280,12 +314,33 @@ void OptionsTabStatus::changeStatusPreset()
 	if(id == -1)
 		return;
 
-	o->sp[d->cb_preset->text(id)].setMessage(d->te_sp->text());
+	StatusPreset sp;
+	sp.setMessage(d->te_sp->text());
 	if (d->le_sp_priority->text().isEmpty())
-		o->sp[d->cb_preset->text(id)].clearPriority();
+		sp.clearPriority();
 	else
-		o->sp[d->cb_preset->text(id)].setPriority(d->le_sp_priority->text().toInt());
-	o->sp[d->cb_preset->text(id)].setStatus(combomap[d->cb_sp_status->currentItem()]);
+		sp.setPriority(d->le_sp_priority->text().toInt());
+	sp.setStatus(d->cb_sp_status->status());
 
+	QString name = d->cb_preset->text(id);
+	
+	sp.setName(name);
+	if (newPresets.contains(name)) {
+		newPresets[name] = sp;
+	} else {
+		dirtyPresets += name;
+		presets[name] = sp;
+	}
+	
 	emit dataChanged();
 }
+
+void OptionsTabStatus::setStatusPresetWidgetsEnabled(bool enabled)
+{
+	OptStatusUI *d = (OptStatusUI *)w;
+	d->cb_preset->setEnabled(enabled);
+	d->pb_spDelete->setEnabled(enabled);
+	d->cb_sp_status->setEnabled(enabled);
+	d->le_sp_priority->setEnabled(enabled);
+	d->te_sp->setEnabled(enabled);
+}
