@@ -20,6 +20,7 @@
 
 #include "psiactionlist.h"
 #include "iconset.h"
+#include "psioptions.h"
 
 #include <qobject.h>
 
@@ -55,6 +56,10 @@ private:
 	};
 
 	void createActionList( QString name, int id, ActionNames * );
+
+private slots:
+	void optionsChanged();
+
 };
 
 PsiActionList::Private::Private(PsiActionList *_list, PsiCon *_psi)
@@ -70,6 +75,9 @@ PsiActionList::Private::Private(PsiActionList *_list, PsiCon *_psi)
 	createMessage();
 	createChat();
 	createGroupchat();
+
+	connect(PsiOptions::instance(), SIGNAL(optionChanged(const QString&)), SLOT(optionsChanged()));
+	optionsChanged();
 }
 
 PsiActionList::Private::~Private()
@@ -108,29 +116,30 @@ void PsiActionList::Private::createCommon()
 void PsiActionList::Private::createMainWin()
 {
 	{
-		IconActionGroup *viewGroups = new IconActionGroup ( this );
-		viewGroups->setMenuText (tr("View Groups"));
-		viewGroups->setWhatsThis (tr("Toggle visibility of special roster groups"));
-		viewGroups->setUsesDropDown (true);
-		viewGroups->setExclusive (false);
+		IconActionGroup *viewGroups = new IconActionGroup(this);
+		viewGroups->setMenuText(tr("View Groups"));
+		viewGroups->setWhatsThis(tr("Toggle visibility of special roster groups"));
+		viewGroups->setUsesDropDown(true);
+		viewGroups->setExclusive(false);
 
-		IconAction *showOffline = new IconAction (tr("Show Offline Contacts"), "psi/show_offline", tr("Show Offline Contacts"), 0, viewGroups, 0, true);
-		showOffline->setWhatsThis (tr("Toggles visibility of offline contacts in roster"));
+		IconAction *showOffline = new IconAction(tr("Show Offline Contacts"), "psi/show_offline", tr("Show Offline Contacts"), 0, viewGroups, 0, true);
+		showOffline->setWhatsThis(tr("Toggles visibility of offline contacts in roster"));
 
-		IconAction *showAway = new IconAction (tr("Show Away/XA/DnD Contacts"), "psi/show_away", tr("Show Away/XA/DnD Contacts"), 0, viewGroups, 0, true);
-		showAway->setWhatsThis (tr("Toggles visibility of away/xa/dnd contacts in roster"));
+		IconAction *showAway = new IconAction(tr("Show Away/XA/DnD Contacts"), "psi/show_away", tr("Show Away/XA/DnD Contacts"), 0,
+		                                      PsiOptions::instance()->getOption("options.ui.menu.view.show-away").toBool() ? (QObject*)viewGroups : (QObject*)this, 0, true);
+		showAway->setWhatsThis(tr("Toggles visibility of away/xa/dnd contacts in roster"));
 
-		IconAction *showHidden = new IconAction (tr("Show Hidden Contacts"), "psi/show_hidden", tr("Show Hidden Contacts"), 0, viewGroups, 0, true);
-		showHidden->setWhatsThis (tr("Toggles visibility of hidden contacts in roster"));
+		IconAction *showHidden = new IconAction(tr("Show Hidden Contacts"), "psi/show_hidden", tr("Show Hidden Contacts"), 0, viewGroups, 0, true);
+		showHidden->setWhatsThis(tr("Toggles visibility of hidden contacts in roster"));
 
-		IconAction *showAgents = new IconAction (tr("Show Agents/Transports"), "psi/disco", tr("Show Agents/Transports"), 0, viewGroups, 0, true);
-		showAgents->setWhatsThis (tr("Toggles visibility of agents/transports in roster"));
+		IconAction *showAgents = new IconAction(tr("Show Agents/Transports"), "psi/disco", tr("Show Agents/Transports"), 0, viewGroups, 0, true);
+		showAgents->setWhatsThis(tr("Toggles visibility of agents/transports in roster"));
 
-		IconAction *showSelf = new IconAction (tr("Show Self Contact"), "psi/show_self", tr("Show Self Contact"), 0, viewGroups, 0, true);
-		showSelf->setWhatsThis (tr("Toggles visibility of self contact in roster"));
+		IconAction *showSelf = new IconAction(tr("Show Self Contact"), "psi/show_self", tr("Show Self Contact"), 0, viewGroups, 0, true);
+		showSelf->setWhatsThis(tr("Toggles visibility of self contact in roster"));
 
-		IconAction *showStatusMsg = new IconAction (tr("Show Status Messages"), "psi/statusmsg", tr("Show Status Messages"), 0, viewGroups, 0, true);
-		showSelf->setWhatsThis (tr("Toggles visibility of status messages of contacts"));
+		IconAction *showStatusMsg = new IconAction(tr("Show Status Messages"), "psi/statusmsg", tr("Show Status Messages"), 0, viewGroups, 0, true);
+		showSelf->setWhatsThis(tr("Toggles visibility of status messages of contacts"));
 
 		ActionNames actions[] = {
 			{ "view_groups",  viewGroups  },
@@ -143,7 +152,7 @@ void PsiActionList::Private::createMainWin()
 			{ "", 0 }
 		};
 
-		createActionList( tr( "Show Contacts" ), Actions_MainWin, actions );
+		createActionList(tr("Show Contacts"), Actions_MainWin, actions);
 	}
 
 	{
@@ -172,13 +181,13 @@ void PsiActionList::Private::createMainWin()
 
 	{
 		IconAction *add_act = 0;
-		if (!option.lockdown.roster)
+		if (!PsiOptions::instance()->getOption("options.ui.contactlist.lockdown-roster").toBool())
 			add_act = new MAction(IconsetFactory::icon("psi/addContact"), tr("&Add a contact"), 0, psi, this);
 
 		IconAction *lw_act = new MAction(IconsetFactory::icon("psi/xml"), tr("&XML Console"), 2, psi, this);
 
 		IconAction *actDisco = 0;
-		if(!option.lockdown.services)
+		if(!PsiOptions::instance()->getOption("options.ui.contactlist.disable-service-discovery").toBool())
 			actDisco = new MAction(IconsetFactory::icon("psi/disco"), tr("Service &Discovery"), 3, psi, this);
 
 //		IconAction *actReadme = new IconAction (tr("ReadMe"), tr("&ReadMe"), 0, this);
@@ -198,18 +207,19 @@ void PsiActionList::Private::createMainWin()
 
 		IconAction *actNewMessage = new IconAction (tr("New blank message"), "psi/sendMessage", tr("New &blank message"), 0, this);
 		IconAction *actJoinGroupchat = new IconAction (tr("Join Groupchat"), "psi/groupChat", tr("Join &Groupchat"), 0, this);
-		IconAction *actAccountSetup = 0;
-		actAccountSetup = new IconAction (tr("Account Setup"), "psi/account", tr("Acc&ount Setup"), 0, this);
-		IconAction *actOptions = 0;
-		actOptions = new IconAction (tr("Options"), "psi/options", tr("&Options"), 0, this);
+		IconAction *actAccountSetup = new IconAction (tr("Account Setup"), "psi/account", tr("Acc&ount Setup"), 0, this);
+
+		IconAction *actOptions = new IconAction (tr("Options"), "psi/options", tr("&Options"), 0, this);
+		actOptions->setMenuRole(QAction::PreferencesRole);
+
 		IconAction *actToolbars = new IconAction(tr("Configure Toolbars"), "psi/toolbars", tr("Configure Tool&bars"), 0, this);
-		IconAction *actChangeProfile = 0;
-		actChangeProfile = new IconAction (tr("Change Profile"), "psi/profile", tr("&Change profile"), 0, this);
+		IconAction *actChangeProfile = new IconAction (tr("Change Profile"), "psi/profile", tr("&Change profile"), 0, this);
 
 		IconAction *actPlaySounds = new IconAction (tr("Play sounds"), "psi/playSounds", tr("Play &sounds"), 0, this, 0, true);
 		actPlaySounds->setWhatsThis (tr("Toggles whether sound should be played or not"));
 		
 		IconAction *actQuit = new IconAction (tr("Quit"), "psi/quit", tr("&Quit"), 0, this);
+		actQuit->setMenuRole(QAction::QuitRole);
 		actQuit->setWhatsThis (tr("Quits Psi"));
 
 		IconAction *actTip = new IconAction (tr("Tip of the Day"), "psi/tip", tr("&Tip of the Day"), 0, this);
@@ -328,19 +338,27 @@ void PsiActionList::Private::createMainWin()
 		actBugReport->setWhatsThis (tr("Report a Bug (Online)"));
 
 		IconAction *actAbout = new IconAction (tr("About"), "psi/logo_16", tr("&About"), 0, this);
+		actAbout->setMenuRole(QAction::AboutRole);
 
 		IconAction *actAboutQt = new IconAction (tr("About Qt"), tr("About &Qt"), 0, this);
+		actAboutQt->setMenuRole(QAction::AboutQtRole);
+
+		IconAction *actDiagQCAPlugin = new IconAction (tr("Security Plugins"), tr("Security &Plugins"), 0, this);
+
+		IconAction *actDiagQCAKeyStore = new IconAction (tr("Key Storage"), tr("&Key Storage"), 0, this);
 
 		ActionNames actions[] = {
-			{ "help_readme",      actReadme     },
-			{ "help_tip",         actTip        },
-			{ "help_online_help", actOnlineHelp },
-			{ "help_online_wiki", actOnlineWiki },
-			{ "help_online_home", actOnlineHome },
-			{ "help_psi_muc",     actPsiMUC     },
-			{ "help_report_bug",  actBugReport  },
-			{ "help_about",       actAbout      },
-			{ "help_about_qt",    actAboutQt    },
+			{ "help_readme",           actReadme          },
+			{ "help_tip",              actTip             },
+			{ "help_online_help",      actOnlineHelp      },
+			{ "help_online_wiki",      actOnlineWiki      },
+			{ "help_online_home",      actOnlineHome      },
+			{ "help_psi_muc",          actPsiMUC          },
+			{ "help_report_bug",       actBugReport       },
+			{ "help_about",            actAbout           },
+			{ "help_about_qt",         actAboutQt         },
+			{ "help_diag_qcaplugin",   actDiagQCAPlugin  },
+			{ "help_diag_qcakeystore", actDiagQCAKeyStore },
 			{ "", 0 }
 		};
 
@@ -372,6 +390,14 @@ void PsiActionList::Private::createGroupchat()
 {
 }
 
+void PsiActionList::Private::optionsChanged()
+{
+	ActionList *statusList = list->actionList(tr("Status"));
+	statusList->action("status_chat")->setVisible(PsiOptions::instance()->getOption("options.ui.menu.status.chat").toBool());
+	statusList->action("status_xa")->setVisible(PsiOptions::instance()->getOption("options.ui.menu.status.xa").toBool());
+	statusList->action("status_invisible")->setVisible(PsiOptions::instance()->getOption("options.ui.menu.status.invisible").toBool());
+}
+
 //----------------------------------------------------------------------------
 // PsiActionList
 //----------------------------------------------------------------------------
