@@ -19,7 +19,6 @@
  */
 
 #include "iconset.h"
-#include "zip.h"
 
 #include <QObject>
 #include <QFile>
@@ -32,6 +31,7 @@
 #include <QDomDocument>
 #include <QThread>
 #include <QCoreApplication>
+#include <QTextCodec>
 
 #include "anim.h"
 
@@ -45,6 +45,14 @@
 #	include <qca_basic.h>
 #endif
 
+#ifndef NO_ICONSET_ZIP
+#define ICONSET_ZIP
+#endif
+
+#ifdef ICONSET_ZIP
+#	include "zip.h"
+#endif
+
 #include <QApplication>
 #include <Q3Shared> // TODO FIXME: port all Q3Shared classes to QSharedData
 
@@ -289,7 +297,7 @@ public slots:
 public:
 	QString name;
 	QRegExp regExp;
-	QHash<QString, QString> text;
+	QList<IconText> text;
 	QString sound;
 
 	Impix impix;
@@ -441,6 +449,11 @@ void PsiIcon::setImpix(const Impix &impix, bool doDetach)
 		detach();
 
 	d->impix = impix;
+	if ( d->icon ) {
+		delete d->icon;
+		d->icon = 0;
+	}
+
 	emit d->pixmapChanged();
 	emit d->iconModified();
 }
@@ -563,7 +576,7 @@ void PsiIcon::setRegExp(const QRegExp &regExp)
  * \sa setText()
  * \sa regExp()
  */
-const QHash<QString, QString> &PsiIcon::text() const
+const QList<PsiIcon::IconText> &PsiIcon::text() const
 {
 	return d->text;
 }
@@ -572,13 +585,52 @@ const QHash<QString, QString> &PsiIcon::text() const
  * Sets the PsiIcon text to \a t.
  * \sa text()
  */
-void PsiIcon::setText(const QHash<QString, QString> &t)
+void PsiIcon::setText(const QList<PsiIcon::IconText> &t)
 {
 	detach();
 
 	d->text = t;
 }
 
+/**
+ * Returns default icon text that could be used as a default value
+ * for e.g. emoticon selector.
+ */
+QString PsiIcon::defaultText() const
+{
+	if (text().isEmpty())
+		return QString();
+
+	// first, try to get the text by priorities
+	QStringList lang;
+	lang << QString(QTextCodec::locale()).left(2); // most prioritent, is the local language
+	lang << "";                                    // and then the language without name goes (international?)
+	lang << "en";                                  // then real English
+
+	QString str;
+	QStringList::Iterator it = lang.begin();
+	for (; it != lang.end(); ++it) {
+		foreach(IconText t, text()){
+			if (t.lang == *it) {
+				str = t.text;
+				break;
+			}
+		}
+	}
+
+	// if all fails, just get the first text
+	if (str.isEmpty()) {
+		foreach(IconText t, text()){
+			if (!t.text.isEmpty()) {
+				str = t.text;
+				break;
+			}
+		}
+	}
+
+	return str;
+}
+
 /**
  * Returns file name of associated sound.
  * \sa setSound()
@@ -961,10 +1013,12 @@ public:
 		QFileInfo fi(dir);
 		if ( fi.isDir() ) {
 			QFile file ( dir + "/" + fileName );
-			file.open (IO_ReadOnly);
+			if (!file.open(IO_ReadOnly))
+				return ba;
 
 			ba = file.readAll();
 		}
+#ifdef ICONSET_ZIP
 		else if ( fi.extension(false) == "jisp" || fi.extension(false) == "zip" ) {
 			UnZip z(dir);
 			if ( !z.open() )
@@ -976,6 +1030,7 @@ public:
 				z.readFile(n, &ba);
 			}
 		}
+#endif
 
 		return ba;
 	}
@@ -1033,7 +1088,8 @@ public:
 		PsiIcon icon;
 		icon.blockSignals(true);
 
-		QHash<QString, QString> text, graphic, sound, object;
+		QList<PsiIcon::IconText> text;
+		QHash<QString, QString> graphic, sound, object;
 
 		QString name;
 		name.sprintf("icon_%04d", icon_counter++);
@@ -1050,7 +1106,10 @@ public:
 				QString lang = e.attribute("xml:lang");
 				if ( lang.isEmpty() )
 					lang = ""; // otherwise there would be many warnings :-(
-				text.insertMulti(lang, e.text());
+				QString t = e.text();
+				if (!t.isEmpty()) {
+					text.append(PsiIcon::IconText(lang, t));
+				}
 			}
 			else if ( tag == "object" ) {
 				object[e.attribute("mime")] = e.text();
@@ -1131,12 +1190,14 @@ public:
 							}
 					}
 
-					if ( icon.loadFromData( loadData(graphic[*it], dir), isAnimated ) ) {
+					QByteArray ba = loadData(graphic[*it], dir);
+
+					if ( icon.loadFromData( ba, isAnimated ) ) {
 						loadSuccess = true;
 						break;
 					}
 					else {
-						qDebug("Iconset::load(): Couldn't load %s graphic for the %s icon for the %s iconset", (*it).latin1(), name.latin1(), this->name.latin1());
+						qDebug("Iconset::load(): Couldn't load %s (%s) graphic for the %s icon for the %s iconset", (*it).toLatin1().data(), graphic[*it].toLatin1().data(), name.toLatin1().data(), this->name.latin1());
 						loadSuccess = false;
 					}
 				}
@@ -1157,9 +1218,6 @@ public:
 						if ( path.isEmpty() )
 							break;
 
-						if( !QCA::isSupported("sha1") )
-							continue;
-
 						QFileInfo ext(sound[*it]);
 						path += "/" + QCA::Hash("sha1").hashToString(QString(fi.absFilePath() + "/" + *sound[*it]).utf8()) + "." + ext.extension();
 
@@ -1182,23 +1240,16 @@ public:
 			}
 		}
 
-		if ( text.count() )
-		{	// construct RegExp
-			QString regexp;
-			
-			QHashIterator<QString, QString> it( text);
-			while ( it.hasNext() ) {
-				it.next();
-				
-				if ( !regexp.isEmpty() )
-					regexp += '|';
-
-				regexp += QRegExp::escape(it.value());
+		// construct RegExp
+		if ( text.count() ) {
+			QStringList regexp;
+			foreach(PsiIcon::IconText t, text) {
+				regexp += QRegExp::escape(t.text);
 			}
-			
+
 			// make sure there is some form of whitespace on at least one side of the text string
 			//regexp = QString("(\\b(%1))|((%2)\\b)").arg(regexp).arg(regexp);
-			icon.setRegExp ( QRegExp(regexp) );
+			icon.setRegExp ( QRegExp(regexp.join("|")) );
 		}
 
 		icon.blockSignals(false);
